function LoginController(ScreenService, MessageService, General, engine, ApplicationContext, templateManager, metaDataFactory) {
    var self = this;

    const updateToken = (httpRequestResponse) => {
        if(httpRequestResponse.dataset){
            const token = httpRequestResponse.dataset.JWT[0];
            setLocalVar('token', token);
        }
    };


    const setLoginWithAppleButtonVisibility = (widget) => {
        const appleLoginButton = widget.getField('apple_login_button');
        const isIOS = window.cordova && window.cordova.platformId === 'ios';
        const IOSVersion = window.device && parseFloat(window.device.version);
        appleLoginButton.isVisible = isIOS && IOSVersion >= 13;
    };
    
   this.controlaVisibilidade = function(widget, isWeb){
        var applyVisibility = function (hadValidQRCode) {
            var visibleGroup = (hadValidQRCode) ? "login" : "default";
            widget.getField('scan').isVisible = !hadValidQRCode;
            widget.fieldGroups.forEach(function (fieldGroup) {
                fieldGroup.isVisible = (visibleGroup == fieldGroup.name);
            });
        };
        if (getLocalVar('validQRCode') && getLocalVar('qrCodeData')) {
            var hadValidQRCode;
            General.callRestEngine("/validateQrCode", "Row", getLocalVar('qrCodeData')).then(
                function (response) {
                    hadValidQRCode = true;
                    updateToken(response);
                },
                function (response) {
                    updateToken(response);
                    hadValidQRCode = false;
                }
            ).finally(function () {
                applyVisibility(hadValidQRCode);
                setLocalVar('validatedQRCode', hadValidQRCode);
                if(isWeb){
                    //self.loginAnonimous();
                    //setLocalVar("showLogout", false);
                    setLocalVar("showLogout", true);
                }
            });
        } else {
            applyVisibility(false);
            setLocalVar('validatedQRCode', false);
        }
        widget.getField('anonimo').isVisible = getLocalVar('IDPERANON') && getLocalVar('IDPERANON') === 'S';  
        setLoginWithAppleButtonVisibility(widget);
    };
    
    this.onEnter = function (widget) {
        setLocalVar('validatedQRCode', false);
        var url_string = window.location.href;
        var url = new URL(url_string);
        var cript = url.hash.split("params=")[1];
        if(cript) {
            const dns = url.origin + url.pathname.split("/mobile/")[0];
            const newCript = cript.replaceAll("%2F", "/").replaceAll("%2520", "+").replaceAll("%252F", "/");
            ApplicationContext.QRCodeController.qrCodeWeb([dns, newCript]).then(() => {
                this.controlaVisibilidade(widget, true);
            });
        } else {
            widget.container.imageSource = getLocalVar('qrCodeURL') ? window.fileServerPath + getLocalVar('qrCodeURL') : widget.container.defaultImageSource;
            this.controlaVisibilidade(widget);
        }
    };

    this.readQrCode = function (widget, qrCodeData) {
        ScreenService.openWindow('product#qrcode');
    };

    this.validateLogin = async function (widget) {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            if (widget.isValid()) {
                const qrCodeObj = JSON.parse(localStorage.getItem('qrCodeData'));
                var userData = {
                    'USER': widget.currentRow.email,
                    'PW': widget.currentRow.senha,
                    'qrCodeData': qrCodeObj.nrqrcode
                };
                General.callRestEngine("/login", "Row", userData).then(
                    function (response) {
                        updateToken(response);
                        setLocalVar('userData', response.dataset.user);
                        setLocalVar('app_version', response.dataset.app_version);
                        let fcmToken = localStorage.getItem('fcmToken');
                        engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
                        ScreenService.openWindow('product#home');
                    },
                    function (response) {
                        var errorMsg = "Erro";
                        if (response && response.error) {
                            errorMsg = response.error;
                        }
    
                        ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
                    }
                );
            }
        }
    };

    this.isLoggedWithFacebook = () => {
        let fbExpirationTime = localStorage.getItem("fbExpirationTime");
        if (!fbExpirationTime) {
            return false;
        }
        let expirationDate = new Date();
        expirationDate.setSeconds(fbExpirationTime);
        if ((new Date()) > expirationDate) {
            return false;
        }
        return true;
    };

    this._requestFacebookLogin = () => {
        return new Promise((resolve, reject) => {
            facebookConnectPlugin.login(["public_profile", "email"],
                data => resolve(data),
                error => reject(error)
            );
        });
    };

    this._getUserFacebookData = (userID) => {
        return new Promise((resolve, reject) => {
            facebookConnectPlugin.api(userID + "?fields=first_name,last_name,email", ["public_profile", "email"],
                result => resolve(result),
                error => reject(error)
            );
        });
    };

    /**
     * TODO: document this function, handle errors properly
     */
     this.showTosWidget = () => {
        const TOSWidget = templateManager.container.getWidget("tosIframe");
        //const tosURL = getLocalVar('tosURL');
        const tosURL = 'https://drive.google.com/viewerng/viewer?embedded=true&url=' + getLocalVar('tosURL');
        const hideToolbarPage = '#toolbar=0&navpanes=0';
        TOSWidget.sourceUrl = tosURL + hideToolbarPage;
        ScreenService.openPopup(TOSWidget);
     };
     
     this.closeTosWidget = () => {
        localStorage.removeItem('facebookAuth');
        localStorage.removeItem('appleAuth');
        ScreenService.closePopup();
     };
     
    this.loginWithFacebook = () => {
        if ('facebookConnectPlugin' in window) {
            this._requestFacebookLogin().then((authData) => {
                let loginData = {};
                let expirationDate = new Date();
                expirationDate.setSeconds(authData.authResponse.expiresIn);
                loginData.accessToken = authData.authResponse.accessToken;
                loginData.expiresIn = moment(expirationDate).format("DD/MM/YYYY");
                loginData.userID = authData.authResponse.userID;
                this._getUserFacebookData(loginData.userID).then((userData) => {
                    Object.assign(loginData, userData);
                    setLocalVar('facebookAuth', loginData);
                    this.showTosWidget();
                });
            }).catch(err => console.log(err));
        }
    };
    
    this.facebookAuth = (loginData) => {
        General.callRestEngine("/facebookAuth", "Row", loginData).then(response => {
            updateToken(response);
            response.dataset.user.isFacebookConnected = true;
            setLocalVar('userData', response.dataset.user);
            setLocalVar('app_version', response.dataset.app_version);
            localStorage.setItem('fbExpirationTime', loginData.expiresIn);
            let fcmToken = localStorage.getItem('fcmToken');
            engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
            this.closeTosWidget();
            ScreenService.openWindow('product#home');
        }, response => {
            var errorMsg = "Erro";
            if (response && response.error) {
                errorMsg = response.error;
            }
            ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
        });
    };
    
    this.callLoginFaceAppleAuth = () => {
        if (JSON.parse(localStorage.getItem('facebookAuth')) !== null) {
            this.facebookAuth(JSON.parse(localStorage.getItem('facebookAuth')));
        } else {
            this.appleAuth(JSON.parse(localStorage.getItem('appleAuth')));
        }        
    };

    this.loginAnonimous = async () => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            General.callRestEngine("/anonimousAccount", "Row", {}).then(function (response) {
                updateToken(response);
                setLocalVar('userData', response.dataset.user);
                setLocalVar('app_version', response.dataset.app_version);
                let fcmToken = localStorage.getItem('fcmToken');
                engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
                ScreenService.openWindow('product#home');
            }, function (response) {
                var errorMsg = "Erro";
                if (response.data && response.data.error) {
                    errorMsg = response.data.error;
                }
    
                ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
            });
        }
    };

    const _getAppleIDUserData = () => {
        return new Promise((resolve, reject) => {
            window.cordova.plugins.SignInWithApple.signin(
                { requestedScopes: [0, 1] },
                function (succ) {
                    console.log(succ);
                    resolve(succ);
                },
                function (err) {
                    console.log(JSON.stringify(err));
                    reject(err);
                }
            );
        });
    };

    this.loginWithApple = () => {
        //TODO: implement actual apple signin
        // this.loginAnonimous()
        _getAppleIDUserData().then((userData) => {
            const {fullName, user, email, identityToken} = userData;
            
            const authData = {
                userID: user,
                email,
                first_name: fullName.givenName,
                last_name: fullName.familyName,
                accessToken: identityToken,
                expiresIn: null
            };

            setLocalVar('appleAuth', authData);
            this.showTosWidget();

        }).catch(() => {
            MessageService.showMessageByCode("MSG_DENIED_ACCESS_WITH_APPLEID");
        });
        
    };

    this.appleAuth = (authData) => {
        General.callRestEngine("/facebookAuth", "Row", authData).then(response => {
            updateToken(response);
            response.dataset.user.isFacebookConnected = true;
            setLocalVar('userData', response.dataset.user);
            setLocalVar('app_version', response.dataset.app_version);
            // localStorage.setItem('fbExpirationTime', loginData.expiresIn);
            const fcmToken = localStorage.getItem('fcmToken');
            engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
            this.closeTosWidget();
            ScreenService.openWindow('product#home');
        });
    };
    
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register('LoginController', LoginController);
});