// FILE: bower_components/red-ui-fields/dist/rui-fields.js
// FILE: src/assets/js/controllers/ruiDateController.js
function ruiDateController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    var nomeDosMeses = [
        "janeiro",
        "fevereiro",
        "março",
        "abril",
        "maio",
        "junho",
        "julho",
        "agosto",
        "setembro",
        "outubro",
        "novembro",
        "dezembro"
    ];

    var diasDaSemanaAbreviado = ["dom", "seg", "ter", "qua", "qui", "sex", "sab"];

    var container = $element.get()[0];
    var dateInput = container.querySelector('#date-input');
    var calendarMode = 0x1;
    var yearMode = 0x2;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            dateInput.classList.add('rui-has-value');
            var dateArray = fieldValue.split('/'); //[0] == day, [1] == month, [2] == year
            dateContainer.date.day = parseInt(dateArray[0], 10);
            dateContainer.selectedDay = parseInt(dateArray[0], 10);
            dateContainer.date.month = parseInt(dateArray[1], 10) - 1;
            dateContainer.selectedMonth = parseInt(dateArray[1], 10) - 1;
            dateContainer.date.year = parseInt(dateArray[2], 10);
            dateContainer.selectedYear = parseInt(dateArray[2], 10);
            dateContainer.dateWasSelected = true;
            dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
            dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
            var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
            dateContainer.monthDisplay.textContent = monthYearString;
            dateInput.children[1].textContent = fieldValue;
            generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        }
        var label = ScreenService.i18n($scope.field.label);
        if ($scope.field.validations && $scope.field.validations.required) {
            label += " *";
        }
        dateInput.children[0].textContent = label;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            dateInput.classList.add('disabled');
        }
    };


    var dateContainer = container.querySelector('#calendar-container');
    var overlay = container.querySelector('#overlay');
    var modal = container.querySelector('#modalWindow');
    dateContainer.okButton = container.querySelector('#ok');
    dateContainer.cancelButton = container.querySelector('#cancel');
    dateContainer.table = container.querySelector('#calendar-table');
    dateContainer.days = container.querySelector('#days-container');
    dateContainer.monthDisplay = container.querySelector('#month-display');
    dateContainer.previousMonthButton = container.querySelector('#previous-month-button');
    dateContainer.nextMonthButton = container.querySelector('#next-month-button');
    dateContainer.topYearDisplay = container.querySelector('#top-year-display');
    dateContainer.topFullDateDisplay = container.querySelector('#top-full-date-display');
    dateContainer.yearPicker = container.querySelector('#year-picker-container');
    dateContainer.yearPickerOutterWrapper = container.querySelector('#year-picker-outter-wrapper');
    dateContainer.currentSelectedYearSpanInYearPicker = undefined;

    dateContainer.dateWasSelected = false;

    dateContainer.mode = calendarMode;

    dateContainer.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (dateContainer.dateWasSelected) {
            dateInput.classList.add('rui-has-value');
        }
        dateInput.classList.remove('active');
        dateContainer.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        dateInput.classList.add('active');
        dateInput.classList.remove('rui-has-value');
        toggleCalendarMode();
        overlay.classList.add('active');
        modal.classList.add('active');
        generateCalendar(dateContainer.date.month, dateContainer.date.year);
    };

    $scope.field.confirm = function(e) {
        dateContainer.dateWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            ((dateContainer.selectedDay) < 10 ? "0" + (dateContainer.selectedDay) : "" + (dateContainer.selectedDay)) +
            "/" + ((dateContainer.selectedMonth + 1) < 10 ? "0" + (dateContainer.selectedMonth + 1) : "" + (dateContainer.selectedMonth + 1)) +
            "/" + dateContainer.selectedYear;
        dateInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        dateInput.classList.add('rui-has-value');
        dateInput.classList.remove('active');
        dateContainer.validate();
    };


    overlay.onclick = $scope.field.cancel;
    dateInput.onclick = $scope.field.open;
    dateContainer.okButton.onclick = $scope.field.confirm;
    dateContainer.cancelButton.onclick = $scope.field.cancel;


    var dayClickHandler = function(eventInfo) {
        var previousActiveDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        previousActiveDay.classList.remove('active');
        dateContainer.selectedDay = Number.parseInt(eventInfo.target.textContent, 10);
        var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        activeDay.classList.add('active');
        dateContainer.date.day = dateContainer.selectedDay;
        dateContainer.date.month = dateContainer.selectedMonth;
        dateContainer.date.year = dateContainer.selectedYear;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    };

    function generateCalendar( /*number */ month, /*number */ year) /*number: phantomOffset*/ {
        //debugger;
        //dateContainer.table;
        //dateContainer.days.opacity = 0;
        while (dateContainer.days.firstChild) {
            dateContainer.days.removeChild(dateContainer.days.firstChild);
        }

        var phantomDay = document.createElement('span');
        phantomDay.classList.add('phantom-day');
        phantomDay.textContent = '00';

        var firstDayOfWeek = new Date(year, month, 1).getDay();
        dateContainer.phantomDaysOffset = firstDayOfWeek - 1;

        for (var i = 0; i < firstDayOfWeek; i++) {
            dateContainer.days.appendChild(phantomDay.cloneNode());
        }

        var lastDayOfMonth = new Date(year, month + 1, 0).getDate();

        for (var j = 1; j <= lastDayOfMonth; j++) {
            var day = document.createElement('span');
            day.textContent = "" + j; //((i < 10) ? "0" + i : "" + i);
            day.onclick = dayClickHandler;
            dateContainer.days.appendChild(day);
        }

        if (dateContainer.date.month === month &&
            dateContainer.date.year === year) {
            var phantomDaysOffset = new Date(year, month, 1).getDay() - 1;
            var activeDay = dateContainer.days.children[phantomDaysOffset + dateContainer.date.day];
            activeDay.classList.add('active');
        }

    }

    /*monthsToAdd = 1 vai para o proximo mes, e -1 para o anterior*/
    /* retorna o objeto da nova data*/
    var addMonths = function( /*number*/ monthsToAdd) {
        var newDate = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        newDate.setMonth(dateContainer.selectedMonth + monthsToAdd);
        var newString = buildMonthWithYearString(newDate.getMonth(), newDate.getFullYear());
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.selectedYear = newDate.getFullYear();
        dateContainer.selectedMonth = newDate.getMonth();
        dateContainer.selectedDay = newDate.getDate();
        generateCalendar(newDate.getMonth(), newDate.getFullYear());
        return newDate;
    };

    var buildMonthWithYearString = function( /*number */ month, /*number */ year) {
        return nomeDosMeses[month] + " de " + year;
    };

    var buildTopDisplayString = function() /* string */ {
        //return nomeDosMeses[month] + " de " + year;
        var date = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        var nomeDiaDaSemana = diasDaSemanaAbreviado[date.getDay()];
        var nomeDoMesAbreviado = nomeDosMeses[dateContainer.selectedMonth].substr(0, 3);
        return nomeDiaDaSemana + ", " + dateContainer.selectedDay + " de " + nomeDoMesAbreviado;
    };

    var setDateUsingJSDateObject = function( /* Date */ data) {
        dateContainer.date.year = data.getFullYear();
        dateContainer.date.month = data.getMonth();
        dateContainer.date.day = data.getDate();
    };

    dateContainer.previousMonthButton.onclick = function() {
        var newDate = addMonths(-1);
    };

    dateContainer.nextMonthButton.onclick = function() {
        var newDate = addMonths(1);
    };

    var toggleYearMode = function() {
        dateContainer.classList.remove('active');
        dateContainer.yearPickerOutterWrapper.classList.add('active');
        dateContainer.topYearDisplay.classList.add('active');
        dateContainer.topFullDateDisplay.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker.scrollIntoView();
    };

    var toggleCalendarMode = function() {
        dateContainer.classList.add('active');
        dateContainer.yearPickerOutterWrapper.classList.remove('active');
        dateContainer.topYearDisplay.classList.remove('active');
        dateContainer.topFullDateDisplay.classList.add('active');
    };

    var pickYear = function( /* clickEvent */ e) {
        var newYear = e.target.textContent;
        dateContainer.selectedYear = newYear;
        var newString = buildMonthWithYearString(dateContainer.selectedMonth, newYear);
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker = e.target;
        e.target.classList.add('active');
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.topYearDisplay.textContent = newYear;
        dateContainer.date.year = newYear;
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        toggleCalendarMode();
    };

    /*initial setup*/

    var today = new Date();
    var hoje = today.getDate();
    dateContainer.selectedDay = hoje;
    dateContainer.selectedMonth = today.getMonth();
    dateContainer.selectedYear = today.getFullYear();
    dateContainer.date = {
        year: today.getFullYear(),
        day: hoje,
        month: today.getMonth()
    };
    dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
    dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    dateContainer.monthDisplay.textContent = buildMonthWithYearString(today.getMonth(), today.getFullYear());
    generateCalendar(today.getMonth(), today.getFullYear());
    var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
    activeDay.classList.add('active');

    dateContainer.topYearDisplay.onclick = toggleYearMode;
    dateContainer.topFullDateDisplay.onclick = toggleCalendarMode;

    for (var i = 1970; i < 2051; i++) {
        var n = document.createElement("span");
        n.textContent = i;
        dateContainer.yearPicker.appendChild(n);
        n.onclick = pickYear;
    }

    dateContainer.yearPicker.currentYearNode = dateContainer.yearPicker.children[today.getFullYear() - 1970];
    dateContainer.yearPicker.currentYearNode.classList.add('active');
    dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;

    /******* ORIENTATION ********/

    if (window.orientation !== undefined) {
        window.addEventListener("orientationchange", function() {
            detectOrientationAndRotate();
        });
    }

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
    };

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        //
        dateInput.classList.remove('rui-has-value');
        var today = new Date();
        dateContainer.date.day = today.getDate();
        dateContainer.selectedDay = dateContainer.date.day;
        dateContainer.date.month = today.getMonth();
        dateContainer.selectedMonth = dateContainer.date.month;
        dateContainer.date.year = today.getFullYear();
        dateContainer.selectedYear = dateContainer.date.year;
        dateContainer.dateWasSelected = false;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
        var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
        dateContainer.monthDisplay.textContent = monthYearString;
        dateInput.children[1].textContent = '';
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        $scope.fieldRow = '';
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.yearPicker.currentYearNode.classList.add('active');
        dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;
        dateContainer.validate();
    };
}


// FILE: src/assets/js/controllers/ruiDateNativeController.js
function ruiDateNativeController($scope, $element, ZhUtil, ScreenService) {
	var defaultFormat = "YYYY-MM-DD";
	var field = $scope.field;
	var input = $element.get(0).querySelector(".rui-txt-input-text");
	var range = {
		maxDate: field.maxDate,
		minDate: field.minDate
	};

	field.applyDefaultValue = function() {
		if (field.widget.currentRow) {
			field.widget.currentRow[field.name] = typeof field.defaultValue === "undefined" ? "" : ZhUtil.evaluate(field.defaultValue);
			field.currentDate = getValueWithDefaultFormat(field.widget.currentRow[field.name]);
		}
	};

	if(field.widget.currentRow && field.defaultValue) {
		field.applyDefaultValue();
	}

	$scope.getLabel = function(label, validations) {
		label = ScreenService.i18n(label);
		if (validations && Object.keys(validations).length && validations.required) {
			label += ' *';
		}
		return label;
	};
	$scope.formatValue = function() {
		if (field.currentDate) {
			field.widget.currentRow[field.name] = moment(field.currentDate, defaultFormat).format(field.dateFormat);
		} else {
			field.widget.currentRow[field.name] = field.currentDate;
		}
	};

	updateRange(field.minDate, field.maxDate);

	if (Object.getOwnPropertyDescriptor(field, 'maxDate') && !Object.getOwnPropertyDescriptor(field, 'maxDate').get) {
		Object.defineProperty(field, 'maxDate', {
	        set: function(maxDate) {
	            range.maxDate = maxDate;
	            updateRange(null, maxDate);
	        },
	        get: function(newValue) {
	            return range.maxDate;
	        }
	    });
	}

	if (Object.getOwnPropertyDescriptor(field, 'minDate') && !Object.getOwnPropertyDescriptor(field, 'minDate').get) {
		Object.defineProperty(field, 'minDate', {
	        set: function(minDate) {
	            range.minDate = minDate;
	            updateRange(minDate, null);
	        },
	        get: function(newValue) {
	            return range.minDate;
	        }
	    });
	}

	function updateRange(minDate, maxDate) {
		if (maxDate) {
			var max = moment(ZhUtil.evaluate(maxDate), field.dateFormat).format(defaultFormat);
			input.setAttribute("max", max);
		}
		if (minDate) {
			var min = moment(ZhUtil.evaluate(minDate), field.dateFormat).format(defaultFormat);
			input.setAttribute("min", min);
		}
	}

	function getValueWithDefaultFormat(value) {
		return moment(value, field.dateFormat).format(defaultFormat);
	}
}

// FILE: src/assets/js/controllers/ruiFABController.js
function ruiFABController($rootScope, ApplicationContext,eventAggregator, templateManager) {


    function getNewFAB() {
        var FABRoot = document.createElement('span');
        var FABIcon = document.createElement('span');
        var FABAction = {};

        FABIcon.className = "material-icons fab-icon";
        FABIcon.textContent = "pregnant_woman";

        FABRoot.classList.add('fab-root');
        FABRoot.appendChild(FABIcon);

        FABRoot.setIcon = function( /*string*/ iconName) {
            if (typeof iconName !== 'string') {
                console.log('%cfunction setIcon expects parameter 1 ' +
                    'to be of type string. Received ' + typeof iconName + ' instead',
                    'font-size:20px;background-color:red;color:yellow;');
                return "YOUR CALL WAS REJECTED LOSER";
            }
            FABIcon.textContent = iconName;
        };

        FABRoot.setAction = function( /*object*/ action) {
            if (typeof action !== 'object') {
                console.log('%cfunction setIcon expects parameter 1 ' +
                    'to be of type Object. Received ' + typeof action + ' instead',
                    'font-size:20px;background-color:red;color:yellow;');
                return "YOUR CALL WAS REJECTED LOSER";
            }
            FABAction = action;
        };


        FABRoot.id = "rui-FAB";
        FABRoot.addEventListener('click', function() {
            var eventContext = getApplicationContextWithEventArgs(FABAction);
            $rootScope.$eval(FABRoot.code, eventContext);
            $rootScope.$apply();
        });

        return FABRoot;
    }

    var FABElement = getNewFAB();
    var defaultIconName = "sentimentverysatisfied";

    var getApplicationContextWithEventArgs = function(eventOwner) {
        ApplicationContext.args = {
            owner: eventOwner
        };
        return ApplicationContext;
    };


    var modes = [
        "FAB_edit",
        "FAB_view",
        "FAB_list",
        "FAB_checked_rows",
        "FAB"
    ];

    // find the first action with the FAB mode
    var findActionToDisplay = function(widget) {
        var currentWidgetMode = widget.mode;
        var actions = widget.actions;
        if (widget.hasCheckedRows()) {
            for (var j = actions.length; j--;) {
                if (actions[j].showAsAction === 'FAB_checked_rows') {
                    return actions[j];
                }
            }
        } else {
            for (var i = actions.length; i--;) {
                if (actions[i].showAsAction === 'FAB'||
                    actions[i].showAsAction.indexOf('FAB_'+currentWidgetMode) !== -1) {
                    return actions[i];
                }
            }
        }
    };

    //add a watch to check if the active widget has changed.
    //TODO: find a better solution that DOESN'T use ANGULARJS

    var changeFABVisibility = function() {
        if(!templateManager.container){
            return;
        }
        var widget = templateManager.container.getActiveWidget();
        var action = findActionToDisplay(widget);
        // /*if the*/ TODO: add comment here
        FABElement.className = 'fab-root';
        if (action) {
            FABElement.code = action.code;
            FABElement.setIcon(action.ruiIcon || defaultIconName);
            FABElement.setAction(action);
            FABElement.zhWidgetMode = action.showAsAction;
            FABElement.className += ' active';
            if(action.class){
                FABElement.className += " " + action.class;
            }
        }
    };

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget().hasCheckedRows();
        // else , will return undefined
    }, function(hasCheckedRows) {
        changeFABVisibility();
    });

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget();
        // else , will return undefined
    }, function(hasChanged) {
        changeFABVisibility();
    });

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget().mode;
        // else , will return undefined
    }, function(hasChanged) {
        changeFABVisibility();
    });

    var attachFABToZhContainer = function() {
        var containerToInjectFAB = document.querySelector('div.container');
        if(containerToInjectFAB){
            containerToInjectFAB.appendChild(FABElement);
            return true;
        } else {
            return false;
        }
    };

    // attach the button to the container, using the attempt and error strategy
    var setIntervalReference = setInterval(function(){
        if(attachFABToZhContainer()){
            clearInterval(setIntervalReference);
        }
    },50);


}

Configuration(function(ContextRegister) {
    ContextRegister.register("ruiFABController", ruiFABController);
});


// FILE: src/assets/js/controllers/ruiSelectController.js
function ruiSelect($scope,$element,$timeout) {
	var realInput = $scope.field;
	var FAKE_INPUT_NAME = "#rui_fake_" + realInput.name;
	
	function switchDataDown(fakefieldname,open){
		var selectContainer  = $(fakefieldname).parent().parent();
		if(open){
			var newplaceholder = selectContainer.find('label').text();
			selectContainer.addClass('rui-fixed-at-top');
		}else{
			selectContainer.removeClass('rui-fixed-at-top');
		}
	}
	
	$scope.initAutoCompSelect = function (){
		$timeout(function (){
			var fieldName = realInput.name;
			$('#'+fieldName).on('click',function(){
				switchDataDown(FAKE_INPUT_NAME,true);
				$(FAKE_INPUT_NAME).focus();
			});
		},300);
	};
	
	$scope.closeDataDown = function(){
		switchDataDown(FAKE_INPUT_NAME,false);
		$(FAKE_INPUT_NAME).blur();
	};
	
	$scope.filterComparising = function(row) {
		var filterOptions = realInput.filterOptions;
		var isVisible = false;
		if ($scope.filterQuery) {
			for (var i = 0; i < filterOptions.length; i++) {
				var rowValue = row[filterOptions[i]].toLowerCase();
				rowValue = rowValue.latinize();
				var filterValue = $scope.filterQuery.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					isVisible = true;
					break;
				}
			}
		} else {
			isVisible = true;
		}
		return isVisible;
	};
	
	$scope.chooseAutoComplete = function(row) {
		var outData = realInput.outData;
		if (outData && Object.keys(outData).length) {
			if (outData && angular.isObject(outData)) {
				for(var attr in outData) {
					realInput.widget.currentRow[outData[attr]] = row[attr];
				}
			}
		} else {
			var descriptionField = realInput.descriptionField || realInput.name;
			realInput.widget.currentRow[realInput.name] = row[descriptionField];
			if (realInput.valueField) {
				realInput.widget.currentRow[realInput.valueField] = row[realInput.valueField];
			}
		}
		$scope.closeDataDown();
	};
	
	$scope.cleanField = function() {
		$scope.filterQuery = "";
		$element.find(FAKE_INPUT_NAME).val($scope.filterQuery);
		$(FAKE_INPUT_NAME).focus();
	};
	
	$scope.cleanSelection = function() {
		realInput.widget.currentRow[realInput.name] = '';
	};
}

// FILE: src/assets/js/controllers/ruiTextController.js
function ruiTextController($scope, ScreenService) {
	var defaultFormat = {
		time: "HH:mm"
	};
	var types = {
		time: "time"
	};

	$scope.initField = function() {
		console.log($('#' + $scope.field.name));
	};

	$scope.getFieldVal = function(fieldName) {
		return $("#" + fieldName).val();
	};
	$scope.getLabel = function(label, validations) {
		label = ScreenService.i18n(label);
		if (validations && Object.keys(validations).length && validations.required) {
			label += ' *';
		}
		return label;
	};
	$scope.formatValue = function(filedRow, field) {
		var type = field.type;
		if (type === types.time && field.timeFormat) {
			field.widget.currentRow[field.name] = moment(filedRow, defaultFormat.date).format(field.timeFormat);
		}
	};
}

// FILE: src/assets/js/controllers/ruiTimeController.js
function ruiTimeController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            var tempoArray = fieldValue.split(':'); //[0] ==  minuto [1] == hora
            clock.time.hora = parseInt(tempoArray[0]);
            clock.time.minuto = parseInt(tempoArray[1]);
            clock.hourView.children[12].style.opacity = '0';
            clock.hourView.children[clock.time.hora].style.opacity = '1';
            clock.minuteView.children[0].style.opacity = '0';
            clock.minuteView.children[clock.time.minuto].style.opacity = '1';
            timeInput.classList.add('rui-has-value');
            clock.hourWasSelected = true;
            toggleHourMode();
        }
        timeInput.children[1].textContent = fieldValue;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            timeInput.classList.add('disabled');
        }
    };

    var container = $element.get()[0];
    var clock = container.querySelector('#clock-container');
    var overlay = container.querySelector('#overlay');
    var pointerContainer = container.querySelector('#pointer-container'); /*clock.children*/
    var touchArea = container.querySelector('#touch-area');
    var timeInput = container.querySelector('#time-input');

    /* define a label */
    var label = ScreenService.i18n($scope.field.label);
    if ($scope.field.validations && $scope.field.validations.required) {
        label += " *";
    }
    timeInput.children[0].textContent = label;


    clock.minuteDisplay = container.querySelector('#red-minute');
    clock.hourInnerDisplay = container.querySelector('#red-hours-inner');
    clock.hourOutterDisplay = container.querySelector('#red-hours-outter');
    clock.pointer = pointerContainer;
    clock.HOUR_MODE = 1;
    clock.MINUTE_MODE = 2;
    clock.mode = clock.HOUR_MODE;
    clock.time = {
        hora: 12,
        minuto: 0
    };
    $scope.field.time = clock.time;
    clock.detalhesDoCirculoMinuto = {};
    clock.detalhesDoCirculoHora = {};
    clock.activeMinute = 0; /* Posicao do vetor que indica o minuto selecionado no relogio  */
    clock.activeHour = 0; /* Posicao do vetor que indica a hora selecionado no relogio  */
    clock.activeHourArrayPosition = 2; /*contém a posicao (no array de filhos de relogio, que corresponde ...)*/
    clock.hourWasSelected = false;
    clock.hourView = container.querySelector('#hour-display-container');
    clock.minuteView = container.querySelector('#minute-display-container');
    clock.okButton = container.querySelector('#ok');
    clock.cancelButton = container.querySelector('#cancel');

    clock.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    var clickHander = function(e) {
        var rotationDetails = '';
        if (clock.mode == clock.MINUTE_MODE) {
            var previousMinute = clock.time.minuto;
            var minuto = getMinute(e.clientX, e.clientY, clock.detalhesDoCirculoMinuto);
            if (minuto === 60) {
                minuto = 0;
            }
            clock.time.minuto = minuto;
            rotationDetails = "rotate3d(0,0,1," + (minuto * 6) + "deg)";
            requestAnimationFrame(function() {

                pointerContainer.style.transform = rotationDetails;
                clock.minuteView.children[minuto].style.opacity = '1';
                if (previousMinute !== minuto) {
                    clock.minuteView.children[previousMinute].style.opacity = '0';
                    clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
                    if (Number.isInteger(minuto / 5)) {
                        clock.minuteDisplay.children[minuto / 5].children[0].style.color = 'white';
                        clock.activeMinute = minuto / 5;
                    }
                }
            });
        }
        else /* clock.mode = clock.HOUR_MODE*/ {
            /* se o clique for dentro do circulo interno ...*/
            var previousActiveHourArrayPosition = clock.activeHourArrayPosition;
            var hora = getHour(e.clientX, e.clientY, clock.detalhesDoCirculoHora);
            var rawHora = ((hora === 12) ? 0 : hora);
            rotationDetails = "rotate3d(0,0,1," + (hora * 30) + "deg)";
            var previousHour = clock.time.hora;
            /* se o clique for dentro do circulo interno ...*/
            if (isWithinCircumnference(clock.detalhesDoCirculoHora, {
                    x: e.clientX,
                    y: e.clientY
                })) {
                switchToInnerPointer();
                hora = ((hora) ? hora : 12);
                clock.time.hora = hora;
                clock.activeHourArrayPosition = 2;
            }
            else {
                switchToOutterPointer();
                hora = ((hora === 12 || hora === 0) ? 0 : hora + 12);
                clock.time.hora = hora;
                clock.activeHourArrayPosition = 1;
            }
            requestAnimationFrame(function() {
                pointerContainer.style.transform = rotationDetails;
                clock.hourView.children[hora].style.opacity = '1';
                if (previousHour !== hora) {
                    clock.hourView.children[previousHour].style.opacity = '0';
                    clock.children[previousActiveHourArrayPosition].
                    children[clock.activeHour].children[0].style.color = 'black';
                    //if(Number.isInteger(rawHora/5)){
                    clock.children[clock.activeHourArrayPosition].
                    children[rawHora].children[0].style.color = 'white';
                    clock.activeHour = rawHora;
                    //}
                }
            });
        }
    };




    /* se for uma tela de toque, usa o evento padrão para telas de toque*/
    if (('ontouchmove' in window) ||
        (window.navigator.MaxTouchPoints > 0) ||
        (window.navigator.msMaxTouchPoints > 0)) {

        var touchHandler = function(e) {
            //console.log('%cTOCADO','background-color: green; color: yellow');
            //e.preventDefault();
            var touch = {};
            var rotationDetails = '';
            if (clock.mode == clock.MINUTE_MODE) {
                touch = e.targetTouches[0];
                var previousMinute = clock.time.minuto;
                var minuto = getMinute(touch.pageX, touch.pageY, clock.detalhesDoCirculoMinuto);
                if (minuto === 60) {
                    minuto = 0;
                }
                clock.time.minuto = minuto;
                rotationDetails = "rotate3d(0,0,1," + (minuto * 6) + "deg)";
                requestAnimationFrame(function() {
                    pointerContainer.style.transform = rotationDetails;
                    clock.minuteView.children[minuto].style.opacity = '1';
                    if (previousMinute !== minuto) {
                        clock.minuteView.children[previousMinute].style.opacity = '0';
                        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
                        if (Number.isInteger(minuto / 5)) {
                            clock.minuteDisplay.children[minuto / 5].children[0].style.color = 'white';
                            clock.activeMinute = minuto / 5;
                        }
                    }
                });
            }
            else /* clock.mode = clock.HOUR_MODE*/ {
                var previousActiveHourArrayPosition = clock.activeHourArrayPosition;
                touch = e.targetTouches[0];
                var hora = getHour(touch.pageX, touch.pageY, clock.detalhesDoCirculoHora);
                var rawHora = ((hora === 12) ? 0 : hora);
                rotationDetails = "rotate3d(0,0,1," + (hora * 30) + "deg)";
                var previousHour = clock.time.hora;
                /* se o clique for dentro do circulo interno ...*/
                if (isWithinCircumnference(clock.detalhesDoCirculoHora, {
                        x: touch.pageX,
                        y: touch.pageY
                    })) {
                    switchToInnerPointer();
                    hora = ((hora) ? hora : 12);
                    clock.time.hora = hora;
                    clock.activeHourArrayPosition = 2;
                }
                else {
                    switchToOutterPointer();
                    hora = ((hora === 12 || hora === 0) ? 0 : hora + 12);
                    clock.time.hora = hora;
                    clock.activeHourArrayPosition = 1;
                }
                requestAnimationFrame(function() {
                    pointerContainer.style.transform = rotationDetails;
                    clock.hourView.children[hora].style.opacity = '1';
                    if (previousHour !== hora) {
                        clock.hourView.children[previousHour].style.opacity = '0';
                        clock.children[previousActiveHourArrayPosition].
                        children[clock.activeHour].children[0].style.color = 'black';
                        //if(Number.isInteger(rawHora/5)){
                        clock.children[clock.activeHourArrayPosition].
                        children[rawHora].children[0].style.color = 'white';
                        clock.activeHour = rawHora;
                        //}
                    }
                });
            }
        };
        touchArea.ontouchmove = touchHandler;

        touchArea.ontouchend = function(e) {
            touchArea.ontouchmove = null;
            setTimeout(function() {
                touchArea.ontouchmove = touchHandler;
            }, 200);
            toggleMinuteMode();
            return true;
        };

        touchArea.ontouchstart = function(e) {
            //e.preventDefault();
            touchHandler(e);
        };
    }
    else {
        touchArea.onmousedown = function(e) {
            e.preventDefault();
            clickHander(e);
            document.addEventListener("mousemove", clickHander);
        };
        var removeListenerAndSwitchToMinutes = function(e) {
            document.removeEventListener("mousemove", clickHander);
            toggleMinuteMode();
        };
        document.addEventListener('mouseup', removeListenerAndSwitchToMinutes);
    }

    var getMinute = function( /*number*/ x, /*number*/ y,
        /*object {x: pontoX da origem, y: pontoY da origem} */
        detalhesCirculo) {
        var angleRadian = Math.atan2(y - detalhesCirculo.y, x - detalhesCirculo.x);
        var angle = angleRadian * (180 / Math.PI) + 90;
        if (angle < 0) {
            angle += 360;
        }
        return Math.round(angle / 6);
    };


    var getHour = function( /*number*/ x, /*number*/ y,
        /*object {x: pontoX da origem, y: pontoY da origem} */
        detalhesCirculo) {
        var angleRadian = Math.atan2(y - detalhesCirculo.centerPoint.y, x - detalhesCirculo.centerPoint.x);
        var angle = angleRadian * (180 / Math.PI) + 90;
        if (angle < 0) {
            angle += 360;
        }
        return Math.round(angle / 30);
    };


    // document.querySelector('html').appendChild(overlay);
    var modal = container.querySelector('#modalWindow');

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (clock.hourWasSelected) {
            timeInput.classList.add('rui-has-value');
        }
        timeInput.classList.remove('active');
        toggleHourMode();
        clock.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        timeInput.classList.add('active');
        timeInput.classList.remove('rui-has-value');
        overlay.classList.add('active');
        modal.classList.add('active');
        recalculateGeometry();
    };

    $scope.field.confirm = function(e) {
        clock.hourWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            (clock.time.hora < 10 ? "0" + clock.time.hora : clock.time.hora) + ":" +
            (clock.time.minuto < 10 ? "0" + clock.time.minuto : clock.time.minuto);
        timeInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        timeInput.classList.remove('active');
        timeInput.classList.add('rui-has-value');
        //timeInput.classList.remove('active');
        toggleHourMode();
        clock.validate();
    };

    overlay.onclick = $scope.field.cancel;
    timeInput.onclick = $scope.field.open;
    clock.okButton.onclick = $scope.field.confirm;
    clock.cancelButton.onclick = $scope.field.cancel;

    window.addEventListener("orientationchange", function() {
        detectOrientationAndRotate();
    }, false);

    /**
     *  C L O C K  M E T H O D S
     */


    var recalculateGeometry = function() {

        var rect = clock.getBoundingClientRect();
        var raioRelogio = (clock.clientHeight / 2);
        clock.detalhesDoCirculoMinuto = {
            x: rect.left + raioRelogio,
            y: rect.top + raioRelogio
        };
        clock.detalhesDoCirculoHora = {
            centerPoint: {
                x: rect.left + 40 + 90,
                y: rect.top + 40 + 90
            },
            radius: 90
        };
    };

    var toggleMinuteMode = function() {
        if (clock.mode === clock.MINUTE_MODE)
            return;

        clock.minuteDisplay.classList.remove('rui-opacity-hidden');
        clock.hourInnerDisplay.classList.add('rui-opacity-hidden');
        clock.hourOutterDisplay.classList.add('rui-opacity-hidden');
        clock.pointer.classList.remove('rui-inner-pointer');
        clock.mode = clock.MINUTE_MODE;
        clock.minuteView.classList.add('active');
        clock.hourView.classList.remove('active');


        var rotationDetails = "rotate3d(0,0,1," + (clock.time.minuto * 6) + "deg)";
        requestAnimationFrame(function() {
            pointerContainer.style.transform = rotationDetails;
        });
    };

    var toggleHourMode = function() {
        if (clock.mode === clock.HOUR_MODE)
            return;


        // esconde os minutos
        clock.minuteDisplay.classList.add('rui-opacity-hidden');
        // mostra as horas
        clock.hourInnerDisplay.classList.remove('rui-opacity-hidden');
        clock.hourOutterDisplay.classList.remove('rui-opacity-hidden');
        clock.pointer.classList.add('rui-inner-pointer');
        clock.minuteView.classList.remove('active');
        clock.hourView.classList.add('active');
        clock.mode = clock.HOUR_MODE;

        var rotationDetails = "rotate3d(0,0,1," + (clock.time.hora * 30) + "deg)";
        requestAnimationFrame(function() {
            pointerContainer.style.transform = rotationDetails;
            if (clock.time.hora <= 12 && clock.time.hora > 0) {
                clock.pointer.classList.add('rui-inner-pointer');

            }
            else {
                clock.pointer.classList.remove('rui-inner-pointer');
            }
        });
    };

    var switchToInnerPointer = function() {
        clock.pointer.classList.add('rui-inner-pointer');
    };

    var switchToOutterPointer = function() {
        clock.pointer.classList.remove('rui-inner-pointer');
    };

    var distanceBetween = function(pointA, pointB) {
        return (Math.sqrt(Math.pow(pointA.x - pointB.x, 2) + Math.pow(pointA.y - pointB.y, 2)));
    };


    var isWithinCircumnference = function(circleProperties, pointToCheck) {
        return (distanceBetween(circleProperties.centerPoint, pointToCheck) <= circleProperties.radius);
    };

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
        setInterval(recalculateGeometry, 800);
    };

    clock.hourView.onclick = toggleHourMode;
    clock.minuteView.onclick = toggleMinuteMode;


    for (var i = 1; i < 60; i++) {
        var n = clock.minuteView.children[0].cloneNode();
        if (i < 10) {
            i = "0" + i;
        }
        n.textContent = i;
        n.style.opacity = '0';
        clock.minuteView.appendChild(n);
    }

    /* GERA OS MOSTRADORES DE HORA */
    for (var j = 1; j < 24; j++) {
        var node = clock.hourView.children[0].cloneNode();
        if (j < 10) {
            j = "0" + j;
        }
        node.textContent = j;
        node.style.opacity = '0';
        clock.hourView.appendChild(node);
    }
    /* DEFINE A HORA PADRAO COMO 12h*/
    clock.hourView.children[0].style.opacity = '0';
    clock.hourView.children[12].style.opacity = '1';
    /* DEFINE A HORA COMO 'ATIVA' (PARA MOSTRAR A FONTE EM BRANCO)*/
    clock.hourView.classList.add('active');
    /**/

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        timeInput.classList.remove('rui-has-value');

        clock.children[clock.activeHourArrayPosition].children[clock.activeHour].children[0].style.color = 'black';

        clock.activeHourArrayPosition = 2;
        clock.activeHour = 11;
        clock.children[clock.activeHourArrayPosition].children[0].children[0].style.color = 'white';

        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
        clock.activeMinute = 0;
        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'white';


        clock.hourView.children[clock.time.hora].style.opacity = '0';
        clock.time.hora = 12;
        clock.hourView.children[12].style.opacity = '1';

        clock.minuteView.children[clock.time.minuto].style.opacity = '0';
        clock.time.minuto = 0;
        clock.minuteView.children[0].style.opacity = '1';

        clock.hourWasSelected = false;
        timeInput.children[1].textContent = '';
        $scope.fieldRow = '';
        /*força re-renderização*/
        clock.mode = clock.MINUTE_MODE;
        toggleHourMode();
        /*******/

        clock.validate();

    };
}



// FILE: bower_components/red-ui-footer/dist/rui-footer.js
// FILE: src/assets/js/controllers/ruiFooterScreenController.js
function ruiFooterScreenController($scope, $element, eventAggregator, ScreenService) {
	/*
	 * -> footerHeight
	 * Sets the footerHeight in pixels
	 *
	 * -> footerProperties
	 * Contains the footerProperties object from the containers.json file
	 *
	 * -> footerDOM
	 * The DOM Object representing container of this controller
	 *
	 * -> currentWindow
	 * Informs the current opened window's name (in lowercase in order to be valid as window.openWindow() parameter)
	 *
	 * -> currentScrollTop
	 * Informs the current scrollTop property of the current container's DOM element
	 *
	 * -> isHidden
	 * Informs wether the container is currently hidden or not
	 *
	 * -> this.functionsToCallOnContainerOpen
	 * An array that contains the reference for all the functions that should be called by the OnContainerOpen event
	 */
	var footerHeight, footerProperties, footerDOM, currentWindow, currentScrollTop, isHidden;
	this.functionsToCallOnContainerOpen = [];

	/*
	 * Initializes variables and call functions needed for initializing the template
	 * Works like a class constructor
	 *
	 * return void
	 */
	this.initializeFooter = function() {
		footerHeight = 56;
		footerProperties = $scope.templateManager.containers.zeedhi_project.footerProperties;
		footerDOM = $element.get()[0];
		currentScrollTop = 0;
		isHidden = false;

		// var containerElement = document.getElementsByClassName('container')[0];
		// containerElement.classList.add('rui-has-footer');

		if (footerProperties.color == "flat")
			footerDOM.classList.add("rui-footer-flat");

		if (footerProperties.hideOnScrollDown) {
			this.addScrollEventToWindow();
			this.functionsToCallOnContainerOpen.push(this.addScrollEventToWindow.bind(this));
		}

		this.functionsToCallOnContainerOpen.push(this.selectFooterElement.bind(this));

		this.createDOMElementForEachFooterElement();
		this.selectFooterElement();
	};

	/*
	 * Iterates over footerProperties.elements for creating DOM element for each footerElement
	 * This function substiutes the ng-repeat AngularJS directive
	 *
	 * return void
	 */
	this.createDOMElementForEachFooterElement = function() {
		var footerElementDOMTemplate, newFooterElementDOM;

		footerElementDOMTemplate = footerDOM.querySelector("#rui-footer-element");

		for (var i=0; i<footerProperties.elements.length; i++) {
			newFooterElementDOM = footerElementDOMTemplate.cloneNode(true);
			newFooterElementDOM.setAttribute("id", "rui-footer-element-"+footerProperties.elements[i].windowName.toLowerCase());
			newFooterElementDOM.windowName = footerProperties.elements[i].windowName;
			newFooterElementDOM.addEventListener("click", this.changeWindow.bind(this));
			newFooterElementDOM.getElementsByTagName("span")[0].getElementsByTagName("i")[0].innerHTML = footerProperties.elements[i].icon;
			newFooterElementDOM.getElementsByTagName("p")[0].innerHTML = ScreenService.i18n(footerProperties.elements[i].label);

			footerDOM.appendChild(newFooterElementDOM);
		}

		footerDOM.removeChild(footerElementDOMTemplate);
	};

	/*
	 * Applies the selected effect on footerElement with windowName equal to the container's name
	 *
	 * return void
	 */
	this.selectFooterElement = function() {
		var footerElementDOM;

		if (currentWindow) {
			footerElementDOM = footerDOM.querySelector("#rui-footer-element-"+currentWindow.toLowerCase());
			if (footerElementDOM)
				footerElementDOM.classList.remove("selected");
		}

		currentWindow = $scope.templateManager.container.name;
		var newSelectedFooterElement = footerDOM.querySelector("#rui-footer-element-"+currentWindow.toLowerCase());
		if (newSelectedFooterElement) {
			newSelectedFooterElement.classList.add("selected");
		}
	};

	/*
	 * Changes the current window to newly selected one.
	 *
	 * return void
	 */
	this.changeWindow = function(event) {
		var footerElementName;

		event.stopPropagation();

		footerElementName = event.currentTarget.windowName;

		if (footerElementName != currentWindow)
			window.openWindow(footerElementName);
	};

	/*
	 * Adds scroll event to current container.
	 * Changes the container height to occupy the footer space also (but being covered by it when not hidden)
	 *
	 * return void
	 *
	 * TODO: find a better solution than the setTimeout and than adding the scrollEvent to all containers
	 */
	this.addScrollEventToWindow = function() {
		setTimeout(function() {
			var containers = document.querySelectorAll(".container");

			for (var i=0; i<containers.length; i++)
				containers[i].addEventListener("scroll", this.handleScrollEvent.bind(this));

			containers[0].style.height = "auto";
			containers[0].style.height = (containers[0].clientHeight + footerHeight)+"px";
		}.bind(this), 500);
	};

	/*
	 * Receives the event object and decides wether to hide, unhide or do nothing to the footer
	 *
	 * return void
	 */
	this.handleScrollEvent = function(event) {
		if (currentScrollTop != event.currentTarget.scrollTop) {
			if (currentScrollTop > event.currentTarget.scrollTop) {
				if (isHidden)
					this.unhideFooter();
			}
			else if (!isHidden)
				this.hideFooter();

			currentScrollTop = event.currentTarget.scrollTop;
		}
	};

	/*
	 * Hides the footer by setting his height to 0
	 *
	 * return void
	 */
	this.hideFooter = function() {
		footerDOM.style.height = 0;
		isHidden = true;
	};

	/*
	 * Unhides the footer by setting his height back to footerHeight
	 *
	 * return void
	 */
	this.unhideFooter = function() {
		footerDOM.style.height = footerHeight+"px";
		isHidden = false;
	};

	/*
	 * Register this.selectFooterElement and this.addScrollEventToWindow as a callback for onContainerOpen event
	 */
	eventAggregator.subscribe("onContainerOpen", function() {
		for (var i=0; i<this.functionsToCallOnContainerOpen.length; i++)
			this.functionsToCallOnContainerOpen[i]();
	}.bind(this));
}



// FILE: bower_components/red-ui-header/dist/rui-header.js
// FILE: src/assets/js/controllers/ruiHeaderController.js
/**
 * The controller of header template
 * @param {TemplateManager} templateManager - An instance of TemplateManager
 */
Configuration(function(eventEngine) {
	eventEngine.registerEvent("WidgetOnBack", "widgetOnBack");
});

function ruiHeaderController($scope, templateManager, ScreenService) {

	var self = this;

	var showAsActionTypes = {
		always: "always",
		list: "list",
		view: "view",
		edit: "edit",
		checked_rows: "checked_rows",
		never: "never",
		never_list: "never_list",
		never_view: "never_view",
		never_edit: "never_edit",
		never_checked_rows: "never_checked_rows",
		back: "back",
		back_list: "back_list",
		back_view: "back_view",
		back_edit: "back_edit",
		back_checked_rows: "back_checked_rows"
	};
	var searchStatus = {
		opened: "SO",
		closed: "SC",
		notSearching: "NS"
	};


	/**
	 * Retrieves actions conllection length
	 * @param  {String} type - the action type
	 * @return {Integer} Returns the length of actions collection by type
	 */
	this.getActionsLengthByType = function (type) {
		return getActionsByType(type).length;
	};

	/**
	 * Retrieves actions by type
	 * @param  {String} type - the action type
	 * @return {Array} Array of actions by type
	 */
	this.getActionsByType = getActionsByType;
	function getActionsByType(type) {
		return getActions().filter(function (action) {
			return findShowAsAction(type, action) && action.isVisible;
		});
	}

	function findShowAsAction (types, action) {
		var exist = false;
		types = types.split('|');
		var asAction = action.showAsAction.split('|');
		for  (var i = asAction.length - 1; i >= 0 && exist === false; i--) {
			exist = showAsActionTypes[types[0]] == asAction[i] ? true : showAsActionTypes[types[1]] == asAction[i] ? true : false;
		}
		return exist;
	}

	this.getActionsVisibleLength = function(nameArrayActions) {
		return $scope[nameArrayActions].filter(function(action) {
			return action.isVisible;
		}).length;
	};

	/**
	 * Toggle dropdown visibility with fast animation
	 */
	this.toggleHeaderDropdown = function () {
		$scope.$root.headerDropdownOpened = !$scope.$root.headerDropdownOpened;
	};
	window.toggleHeaderDropdown = this.toggleHeaderDropdown;

	/**
	 * Retrieves the container's active widget
	 * @return {Widget} Active widget
	 */
	this.getActiveWidget = getActiveWidget;
	function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

	/**
	 * Retrieves active widget's current row of current container
	 * @return {Object} Current row
	 */
	this.getCurrentRow = function () {
		return getActiveWidget().currentRow;
	};

	/**
	 * Execute selected action and toggle dropdown
	 * @param  {Action} action - selected action
	 * @param  {Function} actionClick - Zeedhi action click event
	 */
	this.selectDropdownOption = function (action, actionClick) {
		actionClick(action, getActiveWidget());
	};

	/**
	 * Attach actions and methods in container
	 */
	this.updateContainer = updateContainer;
	function updateContainer() {
		var container = getContainer();
		updateCurrentWidget(container);
		container.eachWidget(function(widget){
			widget.getHeaderActions = getHeaderActions;
		});
	}


	function updateCurrentWidget (container) {
		var widget = getActiveWidget();
		widget.headerActions = widget.headerActions || [];
		widget.headerActions = factoryActions(container, widget.headerActions);
		widget.searchCriteria =  widget.searchCriteria || {};
		updateSearching(widget);
		updateHeaderShadow(widget);
		updateHeaderForTWWI(widget, searchStatus.notSearching);
	}
	function updateHeaderForTWWI(widget,search){
		var fatherTemp = widget.getParent().template;
		var srcTemp = 'rui-tabs#widget/tabbedWidgetWithImage.html';
		var ruiHeader = $('.rui-header');
		var ruiContainer = $('.container[zh-fixed]');
		if(search ===  searchStatus.notSearching){
			if(fatherTemp === srcTemp){
				ruiHeader.addClass('header-over-tabs');
				ruiContainer.css('top',0);
			}else{
				ruiHeader.removeClass('header-over-tabs');
				ruiHeader.removeClass('new-bg-header');
				ruiContainer.css('top',ruiHeader.css('height'));
			}
		}else if (search ===  searchStatus.opened && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').hide();
			$('.rui-header').removeClass('header-over-tabs');
			$('.rui-header').removeClass('new-bg-header');
			ruiContainer.css('top',ruiHeader.css('height'));
		}
		else if (search ===  searchStatus.closed && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').show();
			$('.rui-header').addClass('header-over-tabs');
			$('.rui-header').addClass('new-bg-header');
			ruiContainer.css('top',0);
		}
	}
	function updateHeaderShadow(widget) {
		if (widget.parent && isTabbedTemplate(widget.parent.template)) {
			$('.rui-header').removeClass('searchShadow');
		} else {
			$('.rui-header').addClass('searchShadow');
		}
	}

	function isTabbedTemplate(template) {
		return template.search('tabbedWidget.html') !== -1 || template.search('tabbedWidgetWithImage.html') !== -1;
	}

	function updateSearching(widget) {
		if (widget.showSearch && widget.headerSearchBy) {
			$scope.searchVal = widget.searchCriteria._ALL;
		}

		if (widget.keepHeaderSearchingMode && searchFilled()) {
			$scope.$root.searching = true;
		} else if (searchFilled() && widget.mode == "list") {
			$scope.$root.searching = true;
		} else {
			$scope.$root.searching = false;
		}
	}
	this.getNeverMode = getNeverMode;
	function getNeverMode (){
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "never_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "never_edit|never";
			} else if(widget.mode === "view") {
				return "never_view|never";
			} else if(widget.mode === "list") {
				return "never_list|never";
			}
		}
	}
	this.getBackMode = getBackMode;
	function getBackMode (){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "back_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "back_edit|back";
			} else if(widget.mode === "view") {
				return "back_view|back";
			} else if(widget.mode === "list") {
				return "back_list|back";
			}
		}
	}
	this.getCenterMode = getCenterMode;
	function getCenterMode(){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "edit|always";
			} else if(widget.mode === "view") {
				return "view|always";
			} else if(widget.mode === "list") {
				return "list|always";
			}
		}
	}

	/**
	 * Build actions applying mobile and default properties
	 * @param  {Container} container - current container
	 * @param  {Array} actions - array of actions to build
	 * @return {Array} An array of builded actions
	 */
	function factoryActions(container, actions) {
		return actions.map(function (action) {
			action = Util.extend(new Action(), action);
			_extendMobileProperties(action);
			action.container = container;
			action.widget = getActiveWidget();
			return action;
		});
	}

	/**
	 * Set mobile properties if it's a mobile device
	 * @param  {Object} obj - action object
	 */
	function _extendMobileProperties(obj) {
		if (Util.isMobile() && obj.mobile) {
			var mobile = obj.mobile;
			Object.keys(mobile).forEach(function extendMobileProperty(propertyToExtend) {
				if (obj.replaceByMobile || !obj[propertyToExtend] || typeof obj[propertyToExtend] !== 'object') {
					obj[propertyToExtend] = mobile[propertyToExtend];
				} else {
					obj[propertyToExtend] = Util.extend(obj[propertyToExtend], mobile[propertyToExtend]);
				}
			});
		}
	}

	/**
	 * Retrieves header actions of current container
	 * @return {Array} An array of actions
	 */
	function getActions() {
		return getActiveWidget().headerActions || [];
	}

	/**
	 * Retrieves current container
	 * @return {Container} Current container
	 */
	function getContainer() {
		return templateManager.container;
	}

	function widgetOnChangeMode(args) {
		if (getContainer()) {
			var widget = getActiveWidget();
			if (widget.mode != "list") {
				$scope.$root.searching = false;
			}
			updateContainer(getContainer());
		}
	}

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().mode : null;
	}, widgetOnChangeMode);

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().hasCheckedRows() : null;
	}, function(args) {
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget() : null;
	}, function(args){
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer();
	}, function(args){
		if (getContainer()){
			updateContainer();
		}
	});
	 
	/**
	 * Change header layout to search mode
	 * @param  {Boolean} type - true: Search mode, false: Normal mode
	 */
	this.toggleHeaderSearch = function(type) {
		var widget = getActiveWidget();
		if(type) {
			$scope.$root.searching = true;
			$('.rui-tabsMobileCont').css('top','-50px');
			$('.rui-TWWI-mainContainer').css('top','-50px');
			setTimeout(function() {
				$('#search-input').attr("placeholder", ScreenService.i18n("Search..."));
				$('#search-input').focus();
				updateHeaderForTWWI(widget, searchStatus.opened);
			}, 150);
		} else {
			if (searchFilled()) {
				clearSearch();
			}
			$('.rui-tabsMobileCont').css('top','0');
			$('.rui-TWWI-mainContainer').css('top','0');
			$('#search-input').attr("placeholder", "");
			$scope.$root.searching = false;
			updateHeaderForTWWI(widget, searchStatus.closed);
		}
	};
	window.toggleHeaderSearch = this.toggleHeaderSearch;

	/**
	 * Update the searchCriteria of active widget
	 */
	this.searchInputChange = searchInputChange;
	function searchInputChange() {
		var inputVal = $scope.searchVal;
		var widget = getActiveWidget();
		var showSearch = widget.showSearch;
		if (showSearch && !widget.headerSearchBy) {
			console.error("ERROR: headerSearchBy property not defined on widget '" + widget.name + "'.");
		} else {
			widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
			if (!Util.isEmptyOrBlank(inputVal)) {
				widget.searchCriteria._ALL = inputVal;
			} else {
				delete widget.searchCriteria._ALL;
			}

			if ($scope.searchTimeout) {
				clearTimeout($scope.searchTimeout);
			}
			$scope.searchTimeout = setTimeout(setSearchFilter.bind(this), 500);
		}
	}

	/**
	 * Set dataSourceFilter of widget according to searchCriteria
	 */
	function setSearchFilter() {
		var widget = getActiveWidget();

		if (!Object.keys(widget.searchCriteria).length) {
			widget.dataSourceFilter = widget.defaultDataSourceFilter || widget.dataSourceFilter || [];
		} else {
			if (!widget.defaultDataSourceFilter) {
				widget.defaultDataSourceFilter = widget.dataSourceFilter || [];
			}
			var name = widget.headerSearchBy.join("|");
			var operator = "LIKE_ALL";
			var value = '%'+widget.searchCriteria._ALL+'%';
			addFilterCondition(widget, name, operator, value);
		}
		widget.reload();
	}

	/**
	 * Add condition to widget's dataSourceFilter
	 * @param {Widget} widget
	 * @param {String} name
	 * @param {String} operator
	 * @param {String} value
	 */
	function addFilterCondition(widget, name, operator, value) {
		var dataSourceFilter = widget.dataSourceFilter || [];
		dataSourceFilter = dataSourceFilter.filter(function(row) {
			return row.name != name;
		});
		// if (!Util.isUndefinedOrNull(inDataSourceFilter[0])) {
		// 	dataSourceFilter.splice(inDataSourceFilter[0], 1);
		// }
		dataSourceFilter.push({
			"name": name,
			"operator": operator,
			"value": value
		});
		widget.dataSourceFilter = dataSourceFilter;
	}

	/**
	 * Manage visibility of clear search icon
	 * @return {Boolean} Status of widget's searchCriteria property
	 */
	this.searchFilled = searchFilled;
	function searchFilled() {
		var widget = getActiveWidget();
		if (widget.searchCriteria && widget.headerSearchBy) {
			return !Util.isEmptyOrBlank(widget.searchCriteria._ALL);
		} else {
			return false;
		}
	}

	/**
	 * Clear value of search input
	 */
	this.clearSearch = clearSearch;
	function clearSearch() {
		$scope.searchVal = "";
		searchInputChange();
	}

	function getHeaderActions(name) {
		if (this.headerActions) {
			return this.headerActions.filter(function (item) {
				return item.name === name;
			}).shift();
		}
	}

	this.getTitle = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();
		var headerTitle = widget.title || container.label;

		if (widget.showCheckedRowsCounterOnHeader && widget.getCheckedRows().length) {
			headerTitle = widget.getCheckedRows().length;
		}

		return headerTitle;
	};

	this.menuActionVisibility = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();

		if (!container.showMenu || widget.getCheckedRows().length) {
			return false;
		} else {
			if (widget.showMenu === undefined) {
				return true;
			} else {
				return widget.showMenu;
			}
		}
	};
}


// FILE: bower_components/red-ui-list/dist/rui-list.js
// FILE: src/assets/js/controllers/FluidListController.js
function FluidListController($scope, $element, ApplicationContext, restEngine) {

	/*
	 * -> fluidList
	 * The widget with this template in the container json
	 *
	 * -> fluidListDOM
	 * The DOM element representing the fluid list template
	 *
	 * -> actionEvents
	 * Array containing the code element from all events with name 'ActionEvent' defined on the jobson.json widget declaration
	 */
	var self = this;
	var fluidListDOM, fluidList;
	this.actionEvents = [];

	/*
	 * Initializes the variables, check if this widget should be visible and calls the createFLuidListElements method.
	 *
	 * returns void
	 */
	this.initializeFluidList = function() {
		fluidList = $scope.widget;
		fluidListDOM = $element.get()[0];

		if (!fluidList.isVisible)
			fluidListDOM.style.visibility = "hidden";

		fluidListDOM.style.columnCount = Math.floor(Math.sqrt(fluidListDOM.clientWidth)/7);

		this.actionEvents = fluidList.events.filter(function(event) {
			return event.name == "ActionEvent";
		});

		pullData().then(createFluidListElements.bind(null, fluidList, fluidListDOM));
	};

	/*
	 * Populates the fluidList.dataSource.data array with data from the backend.
	 * The backend MUST return an object named 'dataset' with an array property named 'data'
	 *
	 * This method orders the data as first element in 0 position, second element in half position, third in 1 position, fourth in half+1 position...
	 * This is due the div columns functionality. Data must be ordered this way for being displayed the first and second element side by side, third and fourth
	 * side by side and so on.
	 *
	 * returns void
	 */
	 var pullData = function() {
		if (!fluidList.dataSource)
			fluidList.dataSource = {};
		if (!fluidList.dataSourceFilter)
			fluidList.dataSourceFilter = [];

		return restEngine.post(fluidList.dataSource.name, {
			"requestType": "FilterData",
			"filter": fluidList.dataSourceFilter
		}).then(function(fluidList, response) {
			var length = response.dataset.data.length;
			for (var i=0, j=0; i<length/2; i++, j+=2) {
				fluidList.dataSource.data[i] = response.dataset.data[j];
				fluidList.dataSource.data[length/2+i] = response.dataset.data[j+1];
			}
			if (length%2 !== 0)
				fluidList.dataSource.data.push(response.dataset.data[length-1]);
			
			// fluidList.dataSource.data = response.dataset.data;
		}.bind(null, fluidList), function(error) {
			console.log("%cFluidList could not retrieve data from given datasource", "background-color: red; color: black;");
		});
	 };

	/*
	 * Iterates over the fluidList.fields array and create a DOM element for each one of them.
	 *
	 * returns void
	 */
	var createFluidListElements = function(/*object*/ fluidList, /*DOM object*/ fluidListDOM) {
		var fluidListElement, fluidListElementTemplate;

		document.querySelector(".container").style.overflowY = "scroll";
		fluidListElementTemplate = fluidListDOM.firstElementChild;

		for (var i=0; i<fluidList.dataSource.data.length; i++) {
			fluidListElement = fluidListElementTemplate.cloneNode(true);

			fluidListElement.classList.add("rui-fluid-list-element");
			fluidListElement.setAttribute("id", fluidList.dataSource.data[i].name);
			fluidListElement.style.height = (fluidList.dataSource.data[i].height * 15 + 100) + "px";
			fluidListElement.lastElementChild.innerHTML = fluidList.dataSource.data[i].label;
			fluidListElement.style.backgroundImage = "url('"+fluidList.dataSource.data[i].image+"')";
			fluidListElement.addEventListener("click", executeActionEvent.bind(self));

			fluidListDOM.appendChild(fluidListElement);
		}

		fluidListDOM.removeChild(fluidListElementTemplate);
	};

	/*
	 * Evaluates the strings in actionEvents.
	 *
	 * returns void
	 */
	var executeActionEvent = function(/*object*/ event) {
		for (var i=0; i<this.actionEvents.length; i++)
			$scope.$eval(
				this.actionEvents[i].code,
				getApplicationContextWithEventArgs(event.currentTarget.getAttribute("id"))
			);
	};

	/*
	 * Adds to ApplicationContext an element called "args" with reference to the event owner.
	 *
	 * returns object
	 */
	var getApplicationContextWithEventArgs = function(/*string*/ eventTargetName) {
		ApplicationContext.args = {
			owner: filterElementsByName(fluidList.dataSource.data, eventTargetName)
		};

		return ApplicationContext;
	};

	/*
	 * Filters the fluidList.dataSource.data array in search of an object with the name passed as parameter.
	 * Returns the first one found.
	 *
	 * returns object
	 */
	var filterElementsByName = function(/*object array*/ array, /*string*/ name) {
		return array.filter(function(element) {
			return element.name == name;
		})[0];
	};
}

// FILE: src/assets/js/controllers/RuiListController.js
/**
 * The controller of including list template
 * @param {ZHPromise} ZHPromise - An instance of ZHPromise.
 */
function RuiListController($scope, $rootScope, ZHPromise, localStorageEngine, templateManager) {

	var previousScroll;
	var container = $("#rui-list-container");
	var self = this;
	var defaultListProperties = {
		showImage: false,
		clearChangedOnReload: true
	};
	$scope.listProperties = $scope.widget.listProperties || defaultListProperties;
	$scope.rows = $scope.widget.dataSource.data;
	$scope.widget.itemsPerPage = $scope.widget.itemsPerPage || $scope.widget.dataSource.itemsPerPage;
    $scope.loadedAllData = false;
    $scope.loadingData = false;
	$scope.showLoadMore = true;
	var overlay = document.querySelector(".rui-list-overlay");

	container.nonbounce();

	this.getContainer = function() {
		return templateManager.container;
	};

	$scope.filterTitle = function (item) {
		if (item.isVisible && item.showOnList && item.showAsTitle) {
			return item;
		}
	};

	$scope.filterSubtitle = function (item) {
		if (item.isVisible && item.showOnList && !item.showAsTitle) {
			return item;
		}
	};

	$scope.noMatchVisibility = function() {
		var widget = self.getActiveWidget();
		return Util.isUndefinedOrNull(widget.searchCriteria) || !Object.keys(widget.searchCriteria).length;
	};

	$scope.searchNoMatchVisibility = function() {
		var widget = self.getActiveWidget();
		return !Util.isUndefinedOrNull(widget.searchCriteria) && Object.keys(widget.searchCriteria).length;
	};

	this.onLongTapRow = function (row) {
		var dataArray = $scope.widget.dataSource.data.filter(function(row) {
			return row.dropdownOpened;
		});
		if (!row.__isSelected && $scope.widget.allowSelectOnLongTap && dataArray.length === 0) {
			var widget = self.getActiveWidget();
			for(var i = 0; i< dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_' + dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
			$scope.widget.selectAll([row]);
			if (window.navigator.vibrate) {
				window.navigator.vibrate(25);
			}
		}
	};

	this.closeDropdown = function(row) {
		event.preventDefault();
		event.stopPropagation();
		var dataArray = $scope.widget.dataSource.data.filter(function(row) {
			return row.dropdownOpened;
		});
		if(dataArray.length > 0) {
			for(var i = 0; i < dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_'+dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
		} else {
			this.setCurrentRow(row);
		}
		overlay.style.display = "none";
	};

	this.setCurrentRow = function (row) {		
		var widget = $scope.widget;		
		if (widget.beforeMoveRow) {		
			event = widget.beforeMoveRow();		
		}		
		if (!event || !event.defaultPrevented) {		
			widget.currentRow = row;		
			if (widget.currentRow && widget.dataSource && widget.dataSource.data) {		
				widget.currentRow.__index = widget.dataSource.data.indexOf(row);		
			}		
			if (templateManager.project.showDescriptionOrder) {		
				widget.computeDescriptionOrder();		
			}		
		}		
		if (!event.cancelBubble) {		
			if (widget.afterMoveRow) {		
				widget.afterMoveRow();		
			}		
		}		
	};

	this.getCheckboxIcon = function(row) {
		if (row.__isSelected) {
			return "checkbox";
		} else {
			return "checkboxoutlineblank";
		}
	};
this.changeColorField = function(field,widget) {
		var classes;
		var currStatus = field.IDSTATUS;
		var widgetfields = widget.fields;

		widgetfields.forEach(function(fieldOfWidget){
			if(fieldOfWidget.name === "IDSTATUS"){
				classes = fieldOfWidget.classesForTitle;	
			}
		});
		
		if(Util.isUndefinedOrNull(classes)){
			return null;
		}
		if(Util.isUndefinedOrNull(classes[currStatus])){
			return classes.APROVADO;	
		}
		return classes[currStatus];
	};
	this.filteredRows = function() {
		var widget = $scope.widget;
		var searchCriteria = widget.searchCriteria || {};
		var data  = widget.dataSource.data || [];

		function filterComparising(row) {
			var isVisible = false;
			if (Util.objectHasAnyValue(searchCriteria)) {
				Object.keys(searchCriteria).forEach(function(filter) {
					if (!isVisible) {
						if (filter === "_ALL") {
							var fields;
							if (widget.headerSearchBy) {
								widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
								fields = widget.headerSearchBy;
							} else {
								fields = getVisibleFields();
							}
							fields.forEach(function(field) {
								if (!isVisible) {
									isVisible = compareValues(row[field], searchCriteria[filter], isVisible);
								}
							});
						} else {
							isVisible = compareValues(row[filter], searchCriteria[filter], isVisible);
						}
					}
				});
			} else {
				isVisible = true;
			}
			return isVisible;
		}

		function getVisibleFields() {
			return widget.fields.map(function(field) {
				if (field.isVisible && field.showOnList) {
					return field.name;
				}
			});
		}

		function compareValues(rowValue, filterValue) {
			if (!Util.isUndefinedOrNull(rowValue)) {
				rowValue = rowValue.toLowerCase();
				rowValue = rowValue.latinize();
				filterValue = filterValue.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					return true;
				}
			}
			return false;
		}
		return data.filter(filterComparising);
	};

	this.toggleImageCheckbox = function(row) {
		var widget = self.getActiveWidget();
		if (widget.listProperties.showCheckbox) {
			this.toggle(row);
		}
	};


	this.openRow = function(row, widget) {
		if (row.__group) return;

		if ((widget.showCheckbox && widget.dataSource.getCheckedRows().length) || widget.allowMultiSelect || widget.hasCheckedRows()) {
			self.toggle(row);
		} else {
			widget.setCurrentRow(row);
		}
		widget.dataSource.updateCheckedRows();
	};

	this.getActiveWidget = function() {
		return self.getContainer().getActiveWidget();
	};

	this.getClosedStatus = function(row) {
		var status = false;
		if (row && row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function(group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};

	this.toggle = function(row) {
		var widget = $scope.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			var index = getIndex(row);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}
			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}
		}
	};

	function getIndex(row) {
		return $scope.widget.dataSource.checkedRows.indexOf(row);
	}

	this.toggleDropdown = function(row, e) {
		if (!$scope.widget.hasCheckedRows()) {
			var dataArray = $scope.widget.dataSource.data.filter(function(row) {
				return row.dropdownOpened === true;
			});

			if(dataArray.length < 1) {
				var htmlElementTarget = e.target;
				var dropdownContent = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild.firstElementChild;
				var dropdown = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild;
				var availSpace = screen.availHeight - dropdownContent.getBoundingClientRect().top - (screen.availHeight/15);
				if (availSpace > dropdownContent.offsetHeight) {
					dropdownContent.classList.remove('list-dropdown-last');
				} else {
					dropdownContent.classList.add('list-dropdown-last');
				}
				if(!($scope.widget.hasCheckedRows())) {
					row.dropdownOpened = !row.dropdownOpened;
					if(row.dropdownOpened) {
						dropdown.classList.add('list-dropdown-opened');
					} else {
						dropdown.classList.remove('list-dropdown-opened');
					}
				}
				overlay.style.display = "block";
			} else {
				for(var i=0; i<$scope.widget.dataSource.data.length; i++) {
					$scope.widget.dataSource.data[i].dropdownOpened=false;
					document.getElementById('dropdown_'+i).parentElement.classList.remove('list-dropdown-opened');
				}
			}
			e.preventDefault();
			e.stopPropagation();
		} else {
			e.stopPropagation();
			self.toggle(row);
		}
	};

	this.onActionClick = function(action, actionClick, row) {
		if(!($scope.widget.hasCheckedRows())) {
			var activeWidget = self.getActiveWidget();
			event.stopPropagation();
			event.preventDefault();
			actionClick(action, activeWidget, row);
		} else {
			self.toggle(row);
		}
		self.closeDropdown(row);
	};

   	/**
     * Provide endless scroll on list container
     */
    this.infiniteScroll = function() {
        var scrollHeight = container[0].scrollHeight;
        var scrollPosition = container.height() + container.scrollTop();
        previousScroll = previousScroll || scrollPosition;
        if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll < scrollPosition && !$scope.loadingData && !$scope.loadedAllData) {
            this.loadData(true);
        }
        previousScroll = scrollPosition;
    };

    /**
     * Retrieve content from list dataSource
     * @param  {Boolean} apply - Scope apply selector
     */
    this.loadData = function(apply) {
        $scope.loadingData = true;
    	$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
    	$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        this.nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            this.loadMoreVisibility();
            $scope.loadingData = false;
            $('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
	        $('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
        }.bind(this));
    };

    /**
     * Manage load more button visibility
     */
    this.loadMoreVisibility = function() {
    	var listContainer = $('#rui-list-container')[0];
    	if (listContainer) {
	        var scrollHeight = listContainer.scrollHeight + $('.lf-navbar').height();
	        var windowHeight = $(window).height();
	        if (scrollHeight > windowHeight) {
	            $scope.showLoadMore = false;
	        } else {
	            $scope.showLoadMore = true;
	        }

    	}
    };

    /**
     * Retrieve next page of widget's datasource
     * @param  {Widget} widget
     * @return {ZHPromise} Load function callback
     */
    this.nextPage = function(widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    };

    this.getRowImage = function(row) {
    	var title = "";
    	if ($scope.listProperties.imageTitleField && row[$scope.listProperties.imageTitleField]) {
    		title = row[$scope.listProperties.imageTitleField].toString().substr(0,1);
    	}
    	return title;
    };

    this.isActionVisible = function(row, action) {
    	var isVisible = action.condition ? eval(action.condition) : action.isVisible; // jshint ignore:line
    	return isVisible;
    };

    $scope.filterRowActions = function(action) {
    	if (action.showAsAction.indexOf("|") === -1) {
    		return action.showAsAction === "row";
    	} else {
    		var showAsAction = action.showAsAction.split("|");
    		var isVisible = false;
    		for (var i = 0; i < showAsAction.length; i++) {
    			if (showAsAction[i] === "row") {
    				isVisible = true;
    				break;
    			}
    		}
    		return isVisible;
    	}
    };
	/**
	 * Overwrite default reload action of widget
	 * @param {Object[]} additionalFilters - (Optional) Filters to be applied on reload
	 * @param {Boolean} shouldRefreshCurrentRow - (Optional) True if should refresh current row
	 */
	if (!$scope.widget.defaultReload) {
		$scope.widget.defaultReload = angular.copy($scope.widget.reload);
	    $scope.widget.reload = function(additionalFilters, shouldRefreshCurrentRow) {
	    	$scope.loadingData = true;
	    	$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
    		$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
	        if ($scope.listProperties.clearChangedOnReload) {
	        	$scope.widget.dataSource.clearChangedRows();
	        }
	        return $scope.widget.defaultReload(additionalFilters, shouldRefreshCurrentRow).then(function() {
	            container.scrollTop(0);
	            $scope.loadedAllData = false;
	            $scope.loadingData = false;
	            $('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
	        	$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
	            $scope.widget.dataSource.updateChangedRows();
	        }.bind(this));
	    };
	}

	if (!$scope.widget.dataSource.updateChangedRows) {
		var dataSource = $scope.widget.dataSource;
		var ignoreKeys = ['__rightActions', '__leftActions'];

		dataSource._changedRows = [];
		dataSource._changedRowsCache = {};

		dataSource.getHashKey = function(row) {
			var VO = this.getVO();
			var hash = "";
			if (VO && VO.localKey) {
				hash = VO.localKey.reduce(function(hash, key) {
					return row[key] ? hash + "_" + row[key].toString() : hash;
				}, "");
			}
			return hash;
		};

		dataSource.addChangedRows = function (rows) {
			rows = Util.parseToArray(rows);
			rows.forEach(function(row) {
				if (this._changedRows.indexOf(row) === -1) {
					this._changedRows.push(row);
				}
				var hash = this.getHashKey(row);
				if (hash) {
					this._changedRowsCache[hash] = Util.clone(row, ignoreKeys);
				}
			}.bind(this));
		};

		dataSource.clearChangedRows = function() {
			this._changedRows = [];
			this._changedRowsCache = {};
		};

		dataSource.getChangedRows = function() {
			var changedRows = this._changedRows;
			if (this.name) {
				var changedRowsHash = changedRows.map(function(row) {
					return this.getHashKey(row);
				}.bind(this));
				changedRows = changedRows.concat(Object.keys(this._changedRowsCache).filter(function(row) {
					return changedRowsHash.indexOf(row) === -1;
				}.bind(this)).map(function(hash) {
					return this._changedRowsCache[hash];
				}.bind(this)));
			}
			return changedRows;
		};

		dataSource.updateChangedRows = function() {
			if (this.data) {
				this._changedRows = this.data.filter(function(row) {
					var hash = this.getHashKey(row);
					if (this._changedRowsCache.hasOwnProperty(hash)) {
						for (var i in this._changedRowsCache[hash]) {
							row[i] = this._changedRowsCache[hash][i];
						}
					}
					return row.changed;
				}.bind(this));
			} else {
				this.clearChangedRows();
			}
			if (templateManager.container) {
				templateManager.container.updateCheckedRows();
			}
		};
	}

	if (!$scope.widget.cancel) {
		$scope.widget.cancel = this.cancel;
	}

    container.on("scroll", this.infiniteScroll.bind(this));
    $(window).on("resize", this.loadMoreVisibility);
}


// FILE: bower_components/red-ui-sidenav/dist/rui-sidenav.js
// FILE: src/assets/js/controllers/SideNavController.js
function SideNavController($element, $scope, ScreenService, $rootScope, eventAggregator) {


    var transformPropertyName;// = ;
    if(typeof document.body.style.webkitTransform === 'string'){
        transformPropertyName = 'webkitTransform';
    } else if(typeof document.body.style.transform === 'string'){
        transformPropertyName = 'transform';
    } else {
        console.error('TRANSFORM IS NOT SUPPORTED BY THIS BROWSER. \n The sidenav is not supported as well');
    }

    eventAggregator.subscribe('onContainerOpen', function() {
        setupCurrentSelectedOption();
    });

    var setupCurrentSelectedOption = function() {
        /*if this event was still not handled */
        if (!menu.newSelectedOptionWasReStyledProperly) {
            /*if any option was selected before, disable it*/
            if (menu.currentSelectedOption) {
                menu.currentSelectedOption.classList.remove('selected');
                menu.currentSelectedOption.children[0].classList.remove('selected');
                menu.currentSelectedOption.children[1].classList.remove('selected');
            }
            /* find the new option to reStyle searching by the option.name*/
            for (var i = itemListContainer.children.length; i--;) {
                if (itemListContainer.children[i].menuName === $rootScope.templateManager.container.name) {

                    menu.currentSelectedOption = itemListContainer.children[i];
                    menu.currentSelectedOption.classList.add('selected');
                    menu.currentSelectedOption.children[0].classList.add('selected');
                    menu.currentSelectedOption.children[1].classList.add('selected');
                    return;
                }
            }
        }
        /* always specify that the next handler call should reStyle the menu */
        menu.newSelectedOptionWasReStyledProperly = false;
    };

    var htmlContainer = $element.get()[0];
    var self = this;
    menu = htmlContainer.querySelector('.rui-sidenav-menu-container');
    overlay = htmlContainer.querySelector('.rui-sidenav-overlay');
    itemListContainer = htmlContainer.querySelector('.rui-sidenav-itens-list');

    menuContainer = document.querySelector('.something-else');

    /* gain controll over the default toggleMenuButton*/
    var hijackZhSideMenu = function() {
        var toggleMenuButton = document.querySelector("*[zh-click='toggleSideMenu()']");
        if (toggleMenuButton) {
            toggleMenuButton.removeAttribute('zh-click');
            toggleMenuButton.onclick = openRuiSideMenu;
            return true;
        }
        else
            return false;
    };


    $scope.clearListItens = () => {
        while(itemListContainer.lastChild){
            itemListContainer.removeChild(itemListContainer.lastChild);
        }
    };

    $scope.init = function() {
        var rootMenu = $scope.templateManager.project.groupMenu;
        $scope.clearListItens();
        for (var i = 0; i < rootMenu.length; i++)
            renderMenuItens($scope.templateManager.project.groupMenu[i]);
    };
    
    eventAggregator.subscribe('onChangeLanguage', $scope.init);

    var addClickListener = function( /*menuItem : object*/ item, /*HTMLElement*/ htmlElement) {
        if (Util.isFunction(item.click)) {
            htmlElement.addEventListener('click', function(e) {
                menu.preventInteraction();
                $scope.$eval(item.click);
            });
        }
        else if (item.windowName && !item.disabled) {
            htmlElement.addEventListener('click', function(e) {
                menu.preventInteraction();
                ScreenService.openWindow(item.windowName);
                selectMenuOption(e.currentTarget);
                menu.closeMenu(false);
            });
        }
    };

    var selectMenuOption = function( /*HTMLElement*/ newMenuOption) {
        menu.currentSelectedOption.classList.remove('selected');
        menu.currentSelectedOption.children[0].classList.remove('selected');
        menu.currentSelectedOption.children[1].classList.remove('selected');

        newMenuOption.classList.add('selected');
        newMenuOption.children[0].classList.add('selected');
        newMenuOption.children[1].classList.add('selected');

        menu.currentSelectedOption = newMenuOption;
        menu.newSelectedOptionWasReStyledProperly = true;
    };

    menu.previousX = null;
    menu.previousTime = null;

    var openingTouchHandler = function(e) {
        if (window.isInScrollMode) {
            return;
        }
        var absoluteDeltaY = Math.abs(e.targetTouches[0].clientY - menu.previousY);
        if (absoluteDeltaY > 100 &&
            menu.currentTranslation < -135 
        ) {
            window.isInScrollMode = true;
            menu.classList.remove('rui-sidenav-transition-mode');
            menu.doTranslate(-280);
            return;
        }
        if (this.firstX < this.openingLimit && $scope.templateManager.container.showMenu) {
            var newTranslationValue = e.touches[0].clientX -
                applicationContentSurface.firstX -
                280;
            menu.isOpening = true;
            if (newTranslationValue < 0) {
                menu.classList.remove('rui-sidenav-transition-mode');
                menu.doTranslate(newTranslationValue);
            }
            else {
                $rootScope.menuOpen = true;
            }
        }
    };

    var openingTouchStartHandler = function(e) {
        applicationContentSurface.firstX = e.touches[0].clientX;
        menu.previousY = e.targetTouches[0].clientY;
        applicationContentSurface.firstTime = e.timeStamp;
        window.isInScrollMode = false; // bugFix #M49
    };

    var openingTouchEndHandler = function(e) {
        if (menu.isOpening) {
            applicationContentSurface.firstX = 0;
            applicationContentSurface.lastX = 0;
            applicationContentSurface.acumulatedX = 0;
            if (menu.currentTranslation > -135 ||
                (menu.currentTranslation - applicationContentSurface.firstX > -250 &&
                    e.timeStamp - applicationContentSurface.firstTime <= 300)
            ) {
                $rootScope.menuOpen = true;
            }
            else {
                $rootScope.menuOpen = false;
            }
            menu.isOpening = false;
            window.isInScrollMode = false;
        }
    };

    var initAppContentSurface = function() {
        applicationContentSurface = document.querySelector('div.container');
        if (applicationContentSurface === null) {
            return;
        }
        else {

            applicationContentSurface.addEventListener('touchend', openingTouchEndHandler);
            applicationContentSurface.addEventListener('touchmove', openingTouchHandler, false);
            applicationContentSurface.addEventListener('touchstart', openingTouchStartHandler);

            applicationContentSurface.firstX = 51;
            applicationContentSurface.lastX = 0;
            applicationContentSurface.acumulatedX = 0;
            menu.previousY = 0;
            applicationContentSurface.openingLimit = 30;
            applicationContentSurface.lastMenuOpeningTime = 0;
            clearInterval($scope.setIntervalID);
        }
    };

    $scope.initAppContentSurface = function() {
        $scope.setIntervalIterationCount = 0;
        $scope.setIntervalIterationLimit = 50;
        $scope.setIntervalID = setInterval(function() {
            if ($scope.setIntervalIterationCount > $scope.setIntervalIterationLimit)
                clearInterval($scope.setIntervalID);
            $scope.setIntervalIterationCount++;
            initAppContentSurface.call(self);
        }, 100);
    };

    menu.translatePlus = function(value) {
        menu.currentTranslation = Math.round(value + menu.currentTranslation);
        menu.style[transformPropertyName] = 'translate3d(' + menu.currentTranslation + 'px,0,0)';

    };

    menu.doTranslate = function(value) {
        menu.currentTranslation = Math.round(value);
        requestAnimationFrame(function() {
            menu.style[transformPropertyName] = 'translate3d(' + value + 'px,0,0)';
        });
    };

    var touchHandler = function(event) {
        if (window.isInScrollMode) {
            return;
        }
        var absoluteDeltaY = Math.abs(event.targetTouches[0].clientY - menu.previousY);
        if (absoluteDeltaY > 10 &&
            event.targetTouches[0].clientX <= 280 &&
            menu.currentTranslation > -3) {
            window.isInScrollMode = true;
            return;
        }
        var deltaX = event.targetTouches[0].clientX - menu.previousX;

        if (deltaX !== 0) {
            var translation = deltaX * menu.constanteDeAceleracao;
            var newTrans = translation + menu.currentTranslation;

            if (newTrans > 0) {
                menu.classList.add('rui-sidenav-transition-mode');
                menu.doTranslate(0);
                menu.previousTime = event.timeStamp;
                return;
            }

            if (newTrans < -265) {
                menu.classList.add('rui-sidenav-transition-mode');
                menu.doTranslate(-280);
                return;
            }
            menu.translatePlus(translation);
            menu.previousX = event.targetTouches[0].clientX;
        }
        menu.previousTime = event.timeStamp;

    };

    menuContainer.addEventListener('touchstart', function(event) {
        menu.previousTime = event.timeStamp;
        menu.previousX = event.targetTouches[0].clientX;
        menu.previousY = event.targetTouches[0].clientY;
    }, {
        passive: true
    });

    /* conclui o fechamento */
    menuContainer.addEventListener('touchend', function(event) {
        if (menu.currentTranslation <= -132) {
            $rootScope.menuOpen = false;
        }
        else {
           $rootScope.menuOpen = true;
        }
        window.isInScrollMode = false;
    }, {
        passive: true
    });

    menuContainer.addEventListener('touchmove', touchHandler, {
        passive: true
    });
    menu.classList.remove('nonbounce');
    menu.currentTranslation = 0;

    menu.currentTranslation = -280;
    menu.constanteDeAceleracao = 70;

    menu.openMenu = function() {
        menu.classList.add('rui-sidenav-transition-mode');
        requestAnimationFrame(function() {
            menu.style[transformPropertyName] = 'translate3d(' + 0 + 'px,0,0)';
        });
        overlay.classList.add('active');
        $rootScope.menuOpen = true;
    };

    menu.preventInteraction = function() {
        applicationContentSurface.style.pointerEvents = 'none';
        overlay.style.pointerEvents = 'none';
        setTimeout(function() {
            overlay.style.pointerEvents = 'auto';
            applicationContentSurface.style.pointerEvents = 'auto';
        }, 500);
    };

    menu.closeMenu = function( /*boolean*/ dontPreventInteraction) {
        /* prevent interactions with the content surface and then schedules it's reactivation*/
        if (!dontPreventInteraction) {
            menu.preventInteraction();
        }

        /*do the animations*/
        menu.classList.add('rui-sidenav-transition-mode');
        requestAnimationFrame(function() {
            menu.style[transformPropertyName] = 'translate3d(' + -280 + 'px,0,0)';
        });
        overlay.classList.remove('active');
        $rootScope.menuOpen = false;
    };

    /*
    * @deprecated This function is not necessary anymore, 
    * since the setter method of $rootScope.menuOpen will handle the menu animations by itself.  
    */
    menu.toggle = function() {
        if ($rootScope.menuOpen)
            menu.closeMenu();
        else
            menu.openMenu();
    };

    window.isInScrollMode = false;

    window.toggleRuiSideMenu = menu.toggle;
    window.openRuiSideMenu = menu.openMenu;
    $rootScope.menuOpen = false;
    overlay.addEventListener('click', menu.closeMenu);


    var renderMenuItens = function( /*array: object*/ menu) {

        if (menu.title) {
            var title = document.createElement('span');
            title.className = 'rui-sidenav-title';
            title.textContent = ScreenService.i18n(menu.title);
            itemListContainer.appendChild(title);
        }
        var menuItens = menu.menus;
        for (var i = 0; i < menuItens.length; i++) {
           

            /* create contianer */
            var newListItemContainer = document.createElement('span');
            newListItemContainer.className = 'rui-sidenav-item-container';
            addClickListener(menuItens[i], newListItemContainer);

            /* create label */
            var newListItemText = document.createElement('span');
            newListItemText.className = 'rui-sidenav-item-text';
            newListItemText.textContent = ScreenService.i18n(menuItens[i].label);

            /* create icon */
            var newListItemIcon = document.createElement('i');
            newListItemIcon.className = 'material-icons rui-sidenav-item-icon';
            newListItemIcon.textContent = menuItens[i].icon;

            newListItemContainer.appendChild(newListItemIcon);
            newListItemContainer.appendChild(newListItemText);
            newListItemContainer.menuName = menuItens[i].name;
            if (!menuItens[i].aclIsVisible || !menuItens[i].isVisible)
                newListItemContainer.style.display = 'none';
            itemListContainer.appendChild(newListItemContainer);
            
        }
        var divider = document.createElement('span');
        divider.className = 'rui-sidenav-divider';
        itemListContainer.appendChild(divider);
    };

    /*__menuOpen is a internal property used to control the state of the real property menuOpen */
    $rootScope.__menuOpen = false;
    
    Object.defineProperty($rootScope, 'menuOpen', {

        set: function(newValue) {
            this.__menuOpen = newValue;
            if (newValue) {
                menu.classList.add('rui-sidenav-transition-mode');
                menu.currentTranslation = 0;
                requestAnimationFrame(function() {
                    menu.style[transformPropertyName] = 'translate3d(' + 0 + 'px,0,0)';
                });
                overlay.classList.add('active');
            }
            else {
                menu.preventInteraction();
                menu.classList.add('rui-sidenav-transition-mode');
                menu.currentTranslation = -280;
                requestAnimationFrame(function() {
                    menu.style[transformPropertyName] = 'translate3d(' + -280 + 'px,0,0)';
                });
                overlay.classList.remove('active');
            }
        },
        get: function(newValue) {
            return this.__menuOpen;
        }
    });

}


function initiliazeNewMenuFunction(templateManager, eventAggregator) {
    eventAggregator.subscribe("beforeProjectStartup", function() {
        templateManager.project.setMenuItemVisibility =
            function( /*string*/ menuItemName, /* boolean */ visibility) {
                this.getMenu(menuItemName).isVisible = visibility; /* this === project*/

                /* if menu was already rendered, set the css display property */
                if (typeof itemListContainer !== "undefined") {
                    for (var i = itemListContainer.children.length; i--;) {
                        if (itemListContainer.children[i].menuName === menuItemName) {
                            menuItemToChangeVisibility = itemListContainer.children[i];
                            var newVisibility = (visibility ? 'table' : 'none');
                            menuItemToChangeVisibility.style.display = newVisibility;
                            return;
                        }
                    }
                }
            };
    });
}


Configuration(function(ContextRegister) {
    ContextRegister.register("initiliazeNewMenuFunction", initiliazeNewMenuFunction);
});


