# RedUI sidenav

Red UI sidenav is a component which provides a menu template for mobile applications, it can be open by sliding your finger from the left of the screen to right, or by clicking on the menu icon on the screen's top-left corner.
Sidenav will always display a photo of the logged user followed by their name. You can also configurate sidenav to have an item with some action like redirect user to other screen or execute some task like loggout, see [creating your menu](#creating-your-menu) for more information;


---

## Installation

You can start using it by following the steps below:
 - Install [red-ui core](http://code.zeedhi.com/redui-components/red-ui).
 - Add on your project bower.json file this dependency
    - Example:

```JSON
    "dependencies": {
        "red-ui-sidenav": "*"
    }
```
 - Run `bower install`
 - Add the path to rui's sidenav JavaScript and css on your GruntFile.json.
    - Example:
    
```JavaScript
    var srcFiles = {
		libsJS: [
            "bower_components/red-ui-sidenav/dist/rui-sidenav.js"
        ],
        libsCSS: [
        	"bower_components/red-ui-sidenav/dist/rui-sidenav.css"
        ]
	};
```
 - Add the path to rui's sidenav css and less at the less routine on your GruntFile.json.
    - Example:
    
```JavaScript
		less: {
			development: {
				files: {
					'bower_components/red-ui-sidenav/dist/rui-sidenav.css': "bower_components/red-ui-sidenav/src/assets/css/**/*.less"
				}
			}
		}
```
 -  Run `grunt`

---
## Creating your menu 
 
 - Add the path to sideNav.html as a `menuTemplate`'s attribute in your containers.json.
    - Example:
    
```JSON
		menuTemplate: "rui-sidenav#component/sideNav.html"
```

### GroupMenu

A groupMenu is a object which defines a grouping of menu itens. Each group menu will be separeted by a horizontal line with color defined by 'dividerColor' variable.
To define a groupMenu add at your `groupMenu` an object which will represent a group of menu itens. The most important property you need to define in your menu group object is menus which receives an array with each menu item belonging to that group.
- Example:

```JSON
		"groupMenu":[
            {
                "isVisible": true,
                "menus": []
            }
        ]
```

### Menu item

Menu item is the item which will be displayed at your menu and after being clicked will trigger some action like redirect to other page or loggout. To define a menu item add at your `menus` property an object which will represent an menu item. These are the properties you can use insede your menu item.
 
 - name: this property defines the name of the menu item.
 - label: this property defines the text to be displayed on the menu.
 - icon: you can use this property to define a icon to be displayed at the left side of a menu item. The itens attribute must be set as a name of a icon found at [material icons](https://material.io/icons/), white spaces are not allowed, so use _ when the icon's name is composite.
 - events: allows you to trigger a event based on some context.
 - windowName: this property defines the name of the screen to be openned by clicking this item on menu.

Example:

```JSON
    "menus": [
        {
            "label": "text_to_be_displayed",
            "name": "name_of_this_item",
            "isVisible": false,
            "windowName": "screen_to_open",
            "icon": "sentiment_very_satisfied"
        },
        {
            "label": "Logout",
            "name": "logout",
            "isVisible": true,
            "icon": "power_settings_new",
            "events": [
                {
                    "name": "MenuOnClick",
                    "code": "LoginController.logout();"
                }
            ]
        }
    ]
```
 
---
## Sample code

```JSON
    {
        "groupMenu":[
            {
                "isVisible": true,
                "menus": [
                    {
                        "label": "text_to_be_displayed",
                        "name": "name_of_this_item",
                        "isVisible": false,
                        "windowName": "screen_to_open",
                        "icon": "sentiment_very_satisfied"
                    },
                    {
                        "label": "text_to_be_displayed",
                        "name": "name_of_this_item",
                        "isVisible": true,
                        "windowName": "screen_to_open",
                        "icon": "sentiment_very_dissatisfied"
                    },
                    {
                        "label": "Logout",
                        "name": "logout",
                        "isVisible": true,
                        "icon": "power_settings_new",
                        "events": [
                            {
                                "name": "MenuOnClick",
                                "code": "LoginController.logout();"
                            }
                        ]
                    }
                ]
            }
        ]
    }
```

---

```
Made by:
- Bernardo Santos bernardo.santos@teknisa.com
- Gabriel Andrade gabriel.andrade@teknisa.com
- Gabriel Mello gabriel.mello@teknisa.com
- Luiz Filho luiz.filho@teknisa.com
- Matheus Resende matheus.guedes@teknisa.com
- Samuel Cunha samuel.cunha@teknisa.com
- Rodrigo Rodrigues rodrigo.rodrigues@teknisa.com