# RedUI

Red UI is a Framework extension based on Zeedhi.
This application is focused on reusability of a more custom *front-end* for mobile applications. RUI provides templates dedicated for mobile applications, allowing users to have a truly mobile experience.
This is the core component of RedUI, you need to use it together with one of [its components](#rui-components).

---
## Installation

You can start using it by following the steps below:
 - Add on your project bower.json file this dependency
    - Example:

```JSON
    "dependencies": {
        "red-ui": "*"
    }
```
 - Run `bower install`
 
---
## Configurating your gruntFile
 - First you need to add the less routine in your grunt.initConfig:
 
```js
    less: {
    	development: {
    		options: {
    			compress: true,
    			modifyVars: {}
    		},
    		files: {}
    	}
    }
```
 - Then you need to load tasks from less by adding this piece of code:
 
```JS
    grunt.loadNpmTasks('grunt-contrib-less');
```
---

## Advantages
- The main advantage of using RedUI, is that it makes easy to the developer to build an full mobile application, with an very natural interface for the user. 
- RedUI makes use of Less JS. Which allows the developer to, with less coding, do more customization.
- The use of mixins allow with premade css customization,that can be changed and upgraded, making everything standardized, to increase stability and compatibility.

---
## Mixins
Those are like a css object oriented.
Declaration of a mixin element.

```less
    .rui-border (@radius: 0, @struture: solid, @weight: 1px, @color: @inputBorder){
    	border-radius: @radius;
    	border: @struture @weight @color;
    }
```
The code above, if just called like a css attribute in the .less file will take the default params used in the declaration of the mixin class (exemple bellow).
```less
    .main-box{
    	.rui-border;
    }
```
Would set in the element with this class 'main-box' the default attributes of the border.
Same as declaring:
```css
    .main-box{
    	border-radius: 0;
    	border: solid 1px #F5F5F5;
    }
```
But it can be customized:
```less
    .main-box{
    	.rui-border (10deg, dashed, #000);
    }
```
Would generate:
```css
    .main-box{
    	border-radius: 10deg;
    	border: dashed 1px #000;
    }
```
---
## Style variables
You can change customize the style of every template by changing some parameters variables which are the following:
 - darkPrimaryColor
 - primaryColor
 - lightPrimaryColor
 - textIconsColor
 - accentColor
 - primaryText
 - secondaryText
 - dividerColor
 - errorColor
 - fontFamily
 - defaultFontSize

To change one of these variables just add the parameter followed by its value at your gruntfile, inside modifyVars, and run `grunt less` task.
 - Example:
```JS
    modifyVars: {
		darkPrimaryColor: '#D32F2F',
		defaultFontSize: '1rem'
	}
```
---
## RUI components

- [red-ui-dialog](http://code.zeedhi.com/redui-components/red-ui-dialog/tree/master)
- [red-ui-fields](http://code.zeedhi.com/redui-components/red-ui-fields/tree/master)
- [red-ui-footer](http://code.zeedhi.com/redui-components/red-ui-footer/tree/master)
- [red-ui-header](http://code.zeedhi.com/redui-components/red-ui-header/tree/master)
- [red-ui-sidenav](http://code.zeedhi.com/redui-components/red-ui-sidenav/tree/master)
- [red-ui-splash](http://code.zeedhi.com/redui-components/red-ui-splash/tree/master)
- [red-ui-tabs](http://code.zeedhi.com/redui-components/red-ui-tabs/tree/master)

---

```
Made by:
- Bernardo Santos bernardo.santos@teknisa.com
- Gabriel Andrade gabriel.andrade@teknisa.com
- Gabriel Mello gabriel.mello@teknisa.com
- Luiz Filho luiz.filho@teknisa.com
- Matheus Resende matheus.guedes@teknisa.com
- Samuel Cunha samuel.cunha@teknisa.com
- Rodrigo Rodrigues rodrigo.rodrigues@teknisa.com