(function() {

	function CustomizationActionInjector(templateManager, eventAggregator, ApplicationContext) {

		eventAggregator.subscribe("beforeProjectStartup", pushCustomizationAction);

		//------Dependencies
		var zhLayoutPreferences, zhFilterPreferences, zhCustomizationPreferences;

		function getDependencyByCustomizationType(type) {
			var dependency;
			if (type == "layout"){
				dependency = zhLayoutPreferences;
			} else if (type == "filter"){
				dependency = zhFilterPreferences;
			} else if (type =="view"){
				dependency = zhCustomizationPreferences;
			}
			return dependency;
		}

		// function getDefaultVisibilityByTemplate(widget, type) {
		// 	var defaults = [
		// 		{ template: 'widget/list.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/grid.html', 		options: { search: true, filter: true, layout: true, view: true },
		// 		{ template: 'widget/tree/grid.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/kanban.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/crosstab.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/form.html', 		options: { search: false, filter: false, layout: false, view: false },
		// 		{ template: 'widget/dashboard.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'container/dashboard.html', options: { search: true, filter: true, layout: false, view: false }
		// 	];
		// }

		function isCustomizationActionVisible(widget, type) {
			var dependency = getDependencyByCustomizationType(type);
			return dependency && (Util.isUndefined(widget.floatingControl) || 
								  Util.isUndefined(widget.floatingControl.customizationAction) ||
								  widget.floatingControl.customizationAction[type] !== false);
		}

		function pushCustomizationAction() {
			zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
			zhFilterPreferences = ApplicationContext.zhFilterPreferences;
			zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;

			var customFloatingActions = templateManager.project.customFloatingActions;
			var actionExistence = customFloatingActions.filter(function (action) {
				return action.id == "#hashReportsAction#";
			}).shift();
			if (!actionExistence) {
				templateManager.project.customFloatingActions.push({
					"id": "#hashFilterAction#",
					"name": "FilterAction",
					"label": "Search",
					"type": "filter",
					"order": 20,
					"icon": "search",
					"template": "zh-customizations-service#floating-filter-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'filter') || Util.isUndefined(widget.floatingControl) || widget.floatingControl.searchAction !== false; }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashLayoutsAction#",
					"name": "LayoutAction",
					"label": "Layouts",
					"type": "layout",
					"order": 30,
					"icon": "layout",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'layout'); }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashReportsAction#",
					"name": "ReportAction",
					"label": "Views",
					"type": "view",
					"order": 40,
					"icon": "view",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'view'); }
				});
			}
		}
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationActionInjector", CustomizationActionInjector);
	});

})();